@@ -10,22 +10,26 @@
 # pylint: disable=too-many-arguments,import-error,too-many-locals,broad-except
 """`verdi archive` command."""
 from enum import Enum
-from typing import List, Tuple
+import logging
+from pathlib import Path
 import traceback
+from typing import List, Tuple
 import urllib.request
 
 import click
-import tabulate
+from click_spinner import spinner
 
 from aiida.cmdline.commands.cmd_verdi import verdi
 from aiida.cmdline.params import arguments, options
 from aiida.cmdline.params.types import GroupParamType, PathOrUrl
 from aiida.cmdline.utils import decorators, echo
+from aiida.common.exceptions import CorruptStorage, IncompatibleStorageSchema, UnreachableStorage
 from aiida.common.links import GraphTraversalRules
+from aiida.common.log import AIIDA_LOGGER
 
-EXTRAS_MODE_EXISTING = ['keep_existing', 'update_existing', 'mirror', 'none', 'ask']
+EXTRAS_MODE_EXISTING = ['keep_existing', 'update_existing', 'mirror', 'none']
 EXTRAS_MODE_NEW = ['import', 'none']
-COMMENT_MODE = ['newest', 'overwrite']
+COMMENT_MODE = ['leave', 'newest', 'overwrite']
 
 
 @verdi.group('archive')
@@ -33,71 +37,78 @@ def verdi_archive():
     """Create, inspect and import AiiDA archives."""
 
 
-@verdi_archive.command('inspect')
+@verdi_archive.command('version')
+@click.argument('path', nargs=1, type=click.Path(exists=True, readable=True))
+def archive_version(path):
+    """Print the current version of an archive's schema."""
+    # note: this mirrors `cmd_storage:storage_version`
+    # it is currently hardcoded to the `SqliteZipBackend`, but could be generalized in the future
+    from aiida.storage.sqlite_zip.backend import SqliteZipBackend
+    storage_cls = SqliteZipBackend
+    profile = storage_cls.create_profile(path)
+    head_version = storage_cls.version_head()
+    try:
+        profile_version = storage_cls.version_profile(profile)
+    except (UnreachableStorage, CorruptStorage) as exc:
+        echo.echo_critical(f'archive file version unreadable: {exc}')
+    echo.echo(f'Latest archive schema version: {head_version!r}')
+    echo.echo(f'Archive schema version of {Path(path).name!r}: {profile_version!r}')
+
+
+@verdi_archive.command('info')
+@click.argument('path', nargs=1, type=click.Path(exists=True, readable=True))
+@click.option('--detailed', is_flag=True, help='Provides more detailed information.')
+def archive_info(path, detailed):
+    """Summarise the contents of an archive."""
+    # note: this mirrors `cmd_storage:storage_info`
+    # it is currently hardcoded to the `SqliteZipBackend`, but could be generalized in the future
+    from aiida.storage.sqlite_zip.backend import SqliteZipBackend
+    try:
+        storage = SqliteZipBackend(SqliteZipBackend.create_profile(path))
+    except (UnreachableStorage, CorruptStorage) as exc:
+        echo.echo_critical(f'archive file unreadable: {exc}')
+    except IncompatibleStorageSchema as exc:
+        echo.echo_critical(f'archive version incompatible: {exc}')
+    with spinner():
+        try:
+            data = storage.get_info(detailed=detailed)
+        finally:
+            storage.close()
+
+    echo.echo_dictionary(data, sort_keys=False, fmt='yaml')
+
+
+@verdi_archive.command('inspect', hidden=True)
 @click.argument('archive', nargs=1, type=click.Path(exists=True, readable=True))
 @click.option('-v', '--version', is_flag=True, help='Print the archive format version and exit.')
-@click.option('-d', '--data', hidden=True, is_flag=True, help='Print the data contents and exit.')
 @click.option('-m', '--meta-data', is_flag=True, help='Print the meta data contents and exit.')
-def inspect(archive, version, data, meta_data):
+@click.option('-d', '--database', is_flag=True, help='Include information on entities in the database.')
+@decorators.deprecated_command(
+    'This command has been deprecated and will be removed soon. '
+    'Please call `verdi archive version` or `verdi archive info` instead.\n'
+)
+@click.pass_context
+def inspect(ctx, archive, version, meta_data, database):  # pylint: disable=unused-argument
     """Inspect contents of an archive without importing it.
 
-    By default a summary of the archive contents will be printed. The various options can be used to change exactly what
-    information is displayed.
-
-    .. deprecated:: 1.5.0
-        Support for the --data flag
-
+    .. deprecated:: v2.0.0, use `verdi archive version` or `verdi archive info` instead.
     """
-    import dataclasses
-    from aiida.tools.importexport import CorruptArchive, detect_archive_type, get_reader
-
-    reader_cls = get_reader(detect_archive_type(archive))
-
-    with reader_cls(archive) as reader:
-        try:
-            if version:
-                echo.echo(reader.export_version)
-            elif data:
-                # data is an internal implementation detail
-                echo.echo_deprecated('--data is deprecated and will be removed in v2.0.0')
-                echo.echo_dictionary(reader._get_data())  # pylint: disable=protected-access
-            elif meta_data:
-                echo.echo_dictionary(dataclasses.asdict(reader.metadata))
-            else:
-                statistics = {
-                    'Version aiida': reader.metadata.aiida_version,
-                    'Version format': reader.metadata.export_version,
-                    'Computers': reader.entity_count('Computer'),
-                    'Groups': reader.entity_count('Group'),
-                    'Links': reader.link_count,
-                    'Nodes': reader.entity_count('Node'),
-                    'Users': reader.entity_count('User'),
-                }
-                if reader.metadata.conversion_info:
-                    statistics['Conversion info'] = '\n'.join(reader.metadata.conversion_info)
-
-                echo.echo(tabulate.tabulate(statistics.items()))
-        except CorruptArchive as exception:
-            echo.echo_critical(f'corrupt archive: {exception}')
+    if version:
+        ctx.invoke(archive_version, path=archive)
+    elif database:
+        ctx.invoke(archive_info, path=archive, detailed=True)
+    else:
+        ctx.invoke(archive_info, path=archive, detailed=False)
 
 
 @verdi_archive.command('create')
 @arguments.OUTPUT_FILE(type=click.Path(exists=False))
+@options.ALL()
 @options.CODES()
 @options.COMPUTERS()
 @options.GROUPS()
 @options.NODES()
-@options.ARCHIVE_FORMAT(
-    type=click.Choice(['zip', 'zip-uncompressed', 'zip-lowmemory', 'tar.gz', 'null']),
-)
 @options.FORCE(help='Overwrite output file if it already exists.')
-@click.option(
-    '-v',
-    '--verbosity',
-    default='INFO',
-    type=click.Choice(['DEBUG', 'INFO', 'WARNING', 'CRITICAL']),
-    help='Control the verbosity of console logging'
-)
 @options.graph_traversal_rules(GraphTraversalRules.EXPORT.value)
 @click.option(
     '--include-logs/--exclude-logs',
@@ -111,47 +122,55 @@ def inspect(archive, version, data, meta_data):
     show_default=True,
     help='Include or exclude comments for node(s) in export. (Will also export extra users who commented).'
 )
-# will only be useful when moving to a new archive format, that does not store all data in memory
-# @click.option(
-#     '-b',
-#     '--batch-size',
-#     default=1000,
-#     type=int,
-#     help='Batch database query results in sub-collections to reduce memory usage.'
-# )
+@click.option(
+    '--include-authinfos/--exclude-authinfos',
+    default=False,
+    show_default=True,
+    help='Include or exclude authentication information for computer(s) in export.'
+)
+@click.option('--compress', default=6, show_default=True, type=int, help='Level of compression to use (0-9).')
+@click.option(
+    '-b', '--batch-size', default=1000, type=int, help='Stream database rows in batches, to reduce memory usage.'
+)
+@click.option('--test-run', is_flag=True, help='Determine entities to export, but do not create the archive.')
 @decorators.with_dbenv()
 def create(
-    output_file, codes, computers, groups, nodes, archive_format, force, input_calc_forward, input_work_forward,
-    create_backward, return_backward, call_calc_backward, call_work_backward, include_comments, include_logs, verbosity
+    output_file, all_entries, codes, computers, groups, nodes, force, input_calc_forward, input_work_forward,
+    create_backward, return_backward, call_calc_backward, call_work_backward, include_comments, include_logs,
+    include_authinfos, compress, batch_size, test_run
 ):
-    """
-    Export subsets of the provenance graph to file for sharing.
+    """Create an archive from all or part of a profiles's data.
 
-    Besides Nodes of the provenance graph, you can export Groups, Codes, Computers, Comments and Logs.
+    Besides Nodes of the provenance graph, you can archive Groups, Codes, Computers, Comments and Logs.
 
     By default, the archive file will include not only the entities explicitly provided via the command line but also
     their provenance, according to the rules outlined in the documentation.
     You can modify some of those rules using options of this command.
     """
     # pylint: disable=too-many-branches
-    from aiida.common.log import override_log_formatter_context
     from aiida.common.progress_reporter import set_progress_bar_tqdm, set_progress_reporter
-    from aiida.tools.importexport import export, ExportFileFormat, EXPORT_LOGGER
-    from aiida.tools.importexport.common.exceptions import ArchiveExportError
+    from aiida.tools.archive.abstract import get_format
+    from aiida.tools.archive.create import create_archive
+    from aiida.tools.archive.exceptions import ArchiveExportError
 
-    entities = []
+    archive_format = get_format()
 
-    if codes:
-        entities.extend(codes)
+    if all_entries:
+        entities = None
+    else:
+        entities = []
 
-    if computers:
-        entities.extend(computers)
+        if codes:
+            entities.extend(codes)
 
-    if groups:
-        entities.extend(groups)
+        if computers:
+            entities.extend(computers)
 
-    if nodes:
-        entities.extend(nodes)
+        if groups:
+            entities.extend(groups)
+
+        if nodes:
+            entities.extend(nodes)
 
     kwargs = {
         'input_calc_forward': input_calc_forward,
@@ -160,34 +179,22 @@ def create(
         'return_backward': return_backward,
         'call_calc_backward': call_calc_backward,
         'call_work_backward': call_work_backward,
+        'include_authinfos': include_authinfos,
         'include_comments': include_comments,
         'include_logs': include_logs,
         'overwrite': force,
+        'compression': compress,
+        'batch_size': batch_size,
+        'test_run': test_run
     }
 
-    if archive_format == 'zip':
-        export_format = ExportFileFormat.ZIP
-        kwargs.update({'writer_init': {'use_compression': True}})
-    elif archive_format == 'zip-uncompressed':
-        export_format = ExportFileFormat.ZIP
-        kwargs.update({'writer_init': {'use_compression': False}})
-    elif archive_format == 'zip-lowmemory':
-        export_format = ExportFileFormat.ZIP
-        kwargs.update({'writer_init': {'cache_zipinfo': True}})
-    elif archive_format == 'tar.gz':
-        export_format = ExportFileFormat.TAR_GZIPPED
-    elif archive_format == 'null':
-        export_format = 'null'
-
-    if verbosity in ['DEBUG', 'INFO']:
-        set_progress_bar_tqdm(leave=(verbosity == 'DEBUG'))
+    if AIIDA_LOGGER.level <= logging.REPORT:  # pylint: disable=no-member
+        set_progress_bar_tqdm(leave=(AIIDA_LOGGER.level <= logging.INFO))
     else:
         set_progress_reporter(None)
-    EXPORT_LOGGER.setLevel(verbosity)
 
     try:
-        with override_log_formatter_context('%(message)s'):
-            export(entities, filename=output_file, file_format=export_format, **kwargs)
+        create_archive(entities, filename=output_file, archive_format=archive_format, **kwargs)
     except ArchiveExportError as exception:
         echo.echo_critical(f'failed to write the archive file. Exception: {exception}')
     else:
@@ -197,12 +204,9 @@ def create(
 @verdi_archive.command('migrate')
 @arguments.INPUT_FILE()
 @arguments.OUTPUT_FILE(required=False)
-@options.ARCHIVE_FORMAT()
 @options.FORCE(help='overwrite output file if it already exists')
 @click.option('-i', '--in-place', is_flag=True, help='Migrate the archive in place, overwriting the original file.')
-@options.SILENT(hidden=True)
 @click.option(
-    '-v',
     '--version',
     type=click.STRING,
     required=False,
@@ -212,26 +216,10 @@ def create(
     # version inside the function when needed.
     help='Archive format version to migrate to (defaults to latest version).',
 )
-@click.option(
-    '--verbosity',
-    default='INFO',
-    type=click.Choice(['DEBUG', 'INFO', 'WARNING', 'CRITICAL']),
-    help='Control the verbosity of console logging'
-)
-def migrate(input_file, output_file, force, silent, in_place, archive_format, version, verbosity):
-    """Migrate an export archive to a more recent format version.
-
-    .. deprecated:: 1.5.0
-        Support for the --silent flag, replaced by --verbosity
-
-    """
-    from aiida.common.log import override_log_formatter_context
+def migrate(input_file, output_file, force, in_place, version):
+    """Migrate an archive to a more recent schema version."""
     from aiida.common.progress_reporter import set_progress_bar_tqdm, set_progress_reporter
-    from aiida.tools.importexport import detect_archive_type, EXPORT_VERSION
-    from aiida.tools.importexport.archive.migrators import get_migrator, MIGRATE_LOGGER
-
-    if silent is True:
-        echo.echo_deprecated('the --silent option is deprecated, use --verbosity')
+    from aiida.tools.archive.abstract import get_format
 
     if in_place:
         if output_file:
@@ -243,40 +231,36 @@ def migrate(input_file, output_file, force, silent, in_place, archive_format, ve
             'no output file specified. Please add --in-place flag if you would like to migrate in place.'
         )
 
-    if verbosity in ['DEBUG', 'INFO']:
-        set_progress_bar_tqdm(leave=(verbosity == 'DEBUG'))
+    if AIIDA_LOGGER.level <= logging.REPORT:  # pylint: disable=no-member
+        set_progress_bar_tqdm(leave=(AIIDA_LOGGER.level <= logging.INFO))
     else:
         set_progress_reporter(None)
-    MIGRATE_LOGGER.setLevel(verbosity)
+
+    archive_format = get_format()
 
     if version is None:
-        version = EXPORT_VERSION
-
-    migrator_cls = get_migrator(detect_archive_type(input_file))
-    migrator = migrator_cls(input_file)
+        version = archive_format.latest_version
 
     try:
-        with override_log_formatter_context('%(message)s'):
-            migrator.migrate(version, output_file, force=force, out_compression=archive_format)
+        archive_format.migrate(input_file, output_file, version, force=force, compression=6)
     except Exception as error:  # pylint: disable=broad-except
-        if verbosity == 'DEBUG':
+        if AIIDA_LOGGER.level <= logging.DEBUG:
             raise
         echo.echo_critical(
             'failed to migrate the archive file (use `--verbosity DEBUG` to see traceback): '
             f'{error.__class__.__name__}:{error}'
         )
 
-    if verbosity in ['DEBUG', 'INFO']:
-        echo.echo_success(f'migrated the archive to version {version}')
+    echo.echo_success(f'migrated the archive to version {version!r}')
 
 
 class ExtrasImportCode(Enum):
     """Exit codes for the verdi command line."""
-    keep_existing = 'kcl'
-    update_existing = 'kcu'
-    mirror = 'ncu'
-    none = 'knl'
-    ask = 'kca'
+    # pylint: disable=invalid-name
+    keep_existing = ('k', 'c', 'l')
+    update_existing = ('k', 'c', 'u')
+    mirror = ('n', 'c', 'u')
+    none = ('k', 'n', 'l')
 
 
 @verdi_archive.command('import')
@@ -289,6 +273,12 @@ class ExtrasImportCode(Enum):
     help='Discover all URL targets pointing to files with the .aiida extension for these HTTP addresses. '
     'Automatically discovered archive URLs will be downloaded and added to ARCHIVES for importing.'
 )
+@click.option(
+    '--import-group/--no-import-group',
+    default=True,
+    show_default=True,
+    help='Add all imported nodes to the specified group, or an automatically created one'
+)
 @options.GROUP(
     type=GroupParamType(create_if_not_exist=True),
     help='Specify group to which all the import nodes will be added. If such a group does not exist, it will be'
@@ -298,14 +288,13 @@ class ExtrasImportCode(Enum):
     '-e',
     '--extras-mode-existing',
     type=click.Choice(EXTRAS_MODE_EXISTING),
-    default='keep_existing',
+    default='none',
     help='Specify which extras from the export archive should be imported for nodes that are already contained in the '
     'database: '
-    'ask: import all extras and prompt what to do for existing extras. '
+    'none: do not import any extras.'
     'keep_existing: import all extras and keep original value of existing extras. '
     'update_existing: import all extras and overwrite value of existing extras. '
     'mirror: import all extras and remove any existing extras that are not present in the archive. '
-    'none: do not import any extras.'
 )
 @click.option(
     '-n',
@@ -319,11 +308,18 @@ class ExtrasImportCode(Enum):
 @click.option(
     '--comment-mode',
     type=click.Choice(COMMENT_MODE),
-    default='newest',
+    default='leave',
     help='Specify the way to import Comments with identical UUIDs: '
-    'newest: Only the newest Comments (based on mtime) (default).'
+    'leave: Leave the existing Comments in the database (default).'
+    'newest: Use only the newest Comments (based on mtime).'
     'overwrite: Replace existing Comments with those from the import file.'
 )
+@click.option(
+    '--include-authinfos/--exclude-authinfos',
+    default=False,
+    show_default=True,
+    help='Include or exclude authentication information for computer(s) in import.'
+)
 @click.option(
     '--migration/--no-migration',
     default=True,
@@ -331,35 +327,26 @@ class ExtrasImportCode(Enum):
     help='Force migration of archive file archives, if needed.'
 )
 @click.option(
-    '-v',
-    '--verbosity',
-    default='INFO',
-    type=click.Choice(['DEBUG', 'INFO', 'WARNING', 'CRITICAL']),
-    help='Control the verbosity of console logging'
+    '-b', '--batch-size', default=1000, type=int, help='Stream database rows in batches, to reduce memory usage.'
 )
-@options.NON_INTERACTIVE()
+@click.option('--test-run', is_flag=True, help='Determine entities to import, but do not actually import them.')
 @decorators.with_dbenv()
 @click.pass_context
 def import_archive(
-    ctx, archives, webpages, group, extras_mode_existing, extras_mode_new, comment_mode, migration, non_interactive,
-    verbosity
+    ctx, archives, webpages, extras_mode_existing, extras_mode_new, comment_mode, include_authinfos, migration,
+    batch_size, import_group, group, test_run
 ):
-    """Import data from an AiiDA archive file.
+    """Import archived data to a profile.
 
     The archive can be specified by its relative or absolute file path, or its HTTP URL.
     """
     # pylint: disable=unused-argument
-    from aiida.common.log import override_log_formatter_context
     from aiida.common.progress_reporter import set_progress_bar_tqdm, set_progress_reporter
-    from aiida.tools.importexport.dbimport.utils import IMPORT_LOGGER
-    from aiida.tools.importexport.archive.migrators import MIGRATE_LOGGER
 
-    if verbosity in ['DEBUG', 'INFO']:
-        set_progress_bar_tqdm(leave=(verbosity == 'DEBUG'))
+    if AIIDA_LOGGER.level <= logging.REPORT:  # pylint: disable=no-member
+        set_progress_bar_tqdm(leave=(AIIDA_LOGGER.level <= logging.INFO))
     else:
         set_progress_reporter(None)
-    IMPORT_LOGGER.setLevel(verbosity)
-    MIGRATE_LOGGER.setLevel(verbosity)
 
     all_archives = _gather_imports(archives, webpages)
 
@@ -369,15 +356,18 @@ def import_archive(
 
     # Shared import key-word arguments
     import_kwargs = {
+        'import_new_extras': extras_mode_new == 'import',
+        'merge_extras': ExtrasImportCode[extras_mode_existing].value,
+        'merge_comments': comment_mode,
+        'include_authinfos': include_authinfos,
+        'batch_size': batch_size,
+        'create_group': import_group,
         'group': group,
-        'extras_mode_existing': ExtrasImportCode[extras_mode_existing].value,
-        'extras_mode_new': extras_mode_new,
-        'comment_mode': comment_mode,
+        'test_run': test_run,
     }
 
-    with override_log_formatter_context('%(message)s'):
-        for archive, web_based in all_archives:
-            _import_archive(archive, web_based, import_kwargs, migration)
+    for archive, web_based in all_archives:
+        _import_archive_and_migrate(archive, web_based, import_kwargs, migration)
 
 
 def _echo_exception(msg: str, exception, warn_only: bool = False):
@@ -388,12 +378,12 @@ def _echo_exception(msg: str, exception, warn_only: bool = False):
     :param warn_only: If True only print a warning, otherwise calls sys.exit with a non-zero exit status
 
     """
-    from aiida.tools.importexport import IMPORT_LOGGER
+    from aiida.tools.archive.imports import IMPORT_LOGGER
     message = f'{msg}: {exception.__class__.__name__}: {str(exception)}'
     if warn_only:
         echo.echo_warning(message)
     else:
-        IMPORT_LOGGER.debug('%s', traceback.format_exc())
+        IMPORT_LOGGER.info('%s', traceback.format_exc())
         echo.echo_critical(message)
 
 
@@ -403,7 +393,7 @@ def _gather_imports(archives, webpages) -> List[Tuple[str, bool]]:
     :returns: list of (archive path, whether it is web based)
 
     """
-    from aiida.tools.importexport.common.utils import get_valid_import_links
+    from aiida.tools.archive.common import get_valid_import_links
 
     final_archives = []
 
@@ -418,7 +408,7 @@ def _gather_imports(archives, webpages) -> List[Tuple[str, bool]]:
     if webpages is not None:
         for webpage in webpages:
             try:
-                echo.echo_info(f'retrieving archive URLS from {webpage}')
+                echo.echo_report(f'retrieving archive URLS from {webpage}')
                 urls = get_valid_import_links(webpage)
             except Exception as error:
                 echo.echo_critical(
@@ -431,53 +421,53 @@ def _gather_imports(archives, webpages) -> List[Tuple[str, bool]]:
     return final_archives
 
 
-def _import_archive(archive: str, web_based: bool, import_kwargs: dict, try_migration: bool):
+def _import_archive_and_migrate(archive: str, web_based: bool, import_kwargs: dict, try_migration: bool):
     """Perform the archive import.
 
     :param archive: the path or URL to the archive
     :param web_based: If the archive needs to be downloaded first
     :param import_kwargs: keyword arguments to pass to the import function
-    :param try_migration: whether to try a migration if the import raises IncompatibleArchiveVersionError
+    :param try_migration: whether to try a migration if the import raises `IncompatibleStorageSchema`
 
     """
     from aiida.common.folders import SandboxFolder
-    from aiida.tools.importexport import (
-        detect_archive_type, EXPORT_VERSION, import_data, IncompatibleArchiveVersionError
-    )
-    from aiida.tools.importexport.archive.migrators import get_migrator
+    from aiida.tools.archive.abstract import get_format
+    from aiida.tools.archive.imports import import_archive as _import_archive
+
+    archive_format = get_format()
 
     with SandboxFolder() as temp_folder:
 
         archive_path = archive
 
         if web_based:
-            echo.echo_info(f'downloading archive: {archive}')
+            echo.echo_report(f'downloading archive: {archive}')
             try:
-                response = urllib.request.urlopen(archive)
+                with urllib.request.urlopen(archive) as response:
+                    temp_folder.create_file_from_filelike(response, 'downloaded_archive.zip')
             except Exception as exception:
                 _echo_exception(f'downloading archive {archive} failed', exception)
-            temp_folder.create_file_from_filelike(response, 'downloaded_archive.zip')
+
             archive_path = temp_folder.get_abs_path('downloaded_archive.zip')
             echo.echo_success('archive downloaded, proceeding with import')
 
-        echo.echo_info(f'starting import: {archive}')
+        echo.echo_report(f'starting import: {archive}')
         try:
-            import_data(archive_path, **import_kwargs)
-        except IncompatibleArchiveVersionError as exception:
+            _import_archive(archive_path, archive_format=archive_format, **import_kwargs)
+        except IncompatibleStorageSchema as exception:
             if try_migration:
 
-                echo.echo_info(f'incompatible version detected for {archive}, trying migration')
+                echo.echo_report(f'incompatible version detected for {archive}, trying migration')
                 try:
-                    migrator = get_migrator(detect_archive_type(archive_path))(archive_path)
-                    archive_path = migrator.migrate(
-                        EXPORT_VERSION, None, out_compression='none', work_dir=temp_folder.abspath
-                    )
+                    new_path = temp_folder.get_abs_path('migrated_archive.aiida')
+                    archive_format.migrate(archive_path, new_path, archive_format.latest_version, compression=0)
+                    archive_path = new_path
                 except Exception as exception:
                     _echo_exception(f'an exception occurred while migrating the archive {archive}', exception)
 
-                echo.echo_info('proceeding with import of migrated archive')
+                echo.echo_report('proceeding with import of migrated archive')
                 try:
-                    import_data(archive_path, **import_kwargs)
+                    _import_archive(archive_path, archive_format=archive_format, **import_kwargs)
                 except Exception as exception:
                     _echo_exception(
                         f'an exception occurred while trying to import the migrated archive {archive}', exception
