@@ -10,7 +10,6 @@
 # pylint: disable=too-many-arguments
 """`verdi process` command."""
 import click
-
 from kiwipy import communications
 
 from aiida.cmdline.commands.cmd_verdi import verdi
@@ -18,7 +17,7 @@ from aiida.cmdline.params import arguments, options
 from aiida.cmdline.utils import decorators, echo
 from aiida.cmdline.utils.query.calculation import CalculationQueryBuilder
 from aiida.common.log import LOG_LEVELS
-from aiida.manage.manager import get_manager
+from aiida.manage import get_manager
 
 
 @verdi.group('process')
@@ -53,7 +52,9 @@ def process_list(
     to show also the finished ones."""
     # pylint: disable=too-many-locals
     from tabulate import tabulate
-    from aiida.cmdline.utils.common import print_last_process_state_change, check_worker_load
+
+    from aiida.cmdline.utils.common import check_worker_load, print_last_process_state_change
+    from aiida.engine.daemon.client import get_daemon_client
 
     relationships = {}
 
@@ -77,15 +78,19 @@ def process_list(
         echo.echo(tabulated)
         echo.echo(f'\nTotal results: {len(projected)}\n')
         print_last_process_state_change()
-        # Second query to get active process count
-        # Currently this is slow but will be fixed wiith issue #2770
-        # We place it at the end so that the user can Ctrl+C after getting the process table.
-        builder = CalculationQueryBuilder()
-        filters = builder.get_filters(process_state=('created', 'waiting', 'running'))
-        query_set = builder.get_query_set(filters=filters)
-        projected = builder.get_projected(query_set, projections=['pk'])
-        worker_slot_use = len(projected) - 1
-        check_worker_load(worker_slot_use)
+
+        if not get_daemon_client().is_daemon_running:
+            echo.echo_warning('the daemon is not running', bold=True)
+        else:
+            # Second query to get active process count
+            # Currently this is slow but will be fixed with issue #2770
+            # We place it at the end so that the user can Ctrl+C after getting the process table.
+            builder = CalculationQueryBuilder()
+            filters = builder.get_filters(process_state=('created', 'waiting', 'running'))
+            query_set = builder.get_query_set(filters=filters)
+            projected = builder.get_projected(query_set, projections=['pk'])
+            worker_slot_use = len(projected) - 1
+            check_worker_load(worker_slot_use)
 
 
 @verdi_process.command('show')
@@ -139,8 +144,8 @@ def process_call_root(processes):
 @decorators.with_dbenv()
 def process_report(processes, levelname, indent_size, max_depth):
     """Show the log report for one or multiple processes."""
-    from aiida.cmdline.utils.common import get_calcjob_report, get_workchain_report, get_process_function_report
-    from aiida.orm import CalcJobNode, WorkChainNode, CalcFunctionNode, WorkFunctionNode
+    from aiida.cmdline.utils.common import get_calcjob_report, get_process_function_report, get_workchain_report
+    from aiida.orm import CalcFunctionNode, CalcJobNode, WorkChainNode, WorkFunctionNode
 
     for process in processes:
         if isinstance(process, CalcJobNode):
@@ -275,6 +280,7 @@ def process_play(processes, all_entries, timeout, wait):
 def process_watch(processes):
     """Watch the state transitions for a process."""
     from time import sleep
+
     from kiwipy import BroadcastFilter
 
     def _print(communicator, body, sender, subject, correlation_id):  # pylint: disable=unused-argument
@@ -288,7 +294,7 @@ def process_watch(processes):
         echo.echo(f'Process<{sender}> [{subject}|{correlation_id}]: {body}')
 
     communicator = get_manager().get_communicator()
-    echo.echo_info('watching for broadcasted messages, press CTRL+C to stop...')
+    echo.echo_report('watching for broadcasted messages, press CTRL+C to stop...')
 
     for process in processes:
 
@@ -304,7 +310,7 @@ def process_watch(processes):
             sleep(2)
     except (SystemExit, KeyboardInterrupt):
         echo.echo('')  # add a new line after the interrupt character
-        echo.echo_info('received interrupt, exiting...')
+        echo.echo_report('received interrupt, exiting...')
         try:
             communicator.close()
         except RuntimeError:
@@ -336,9 +342,10 @@ def process_actions(futures_map, infinitive, present, past, wait=False, timeout=
     :type timeout: float
     """
     # pylint: disable=too-many-branches
+    from concurrent import futures
+
     import kiwipy
     from plumpy.futures import unwrap_kiwi_future
-    from concurrent import futures
 
     from aiida.manage.external.rmq import CommunicationTimeout
 
@@ -368,7 +375,7 @@ def process_actions(futures_map, infinitive, present, past, wait=False, timeout=
                     echo.echo_error(f'got unexpected response when {present} Process<{process.pk}>: {result}')
 
         if wait and scheduled:
-            echo.echo_info(f"waiting for process(es) {','.join([str(proc.pk) for proc in scheduled.values()])}")
+            echo.echo_report(f"waiting for process(es) {','.join([str(proc.pk) for proc in scheduled.values()])}")
 
             for future in futures.as_completed(scheduled.keys(), timeout=timeout):
                 process = scheduled[future]
