@@ -9,16 +9,18 @@
 ###########################################################################
 """Click parameter types for paths."""
 import os
-# See https://stackoverflow.com/a/41217363/1069467
-import urllib.request
-import urllib.error
 from socket import timeout
+import urllib.error
+import urllib.request
+
 import click
 
+__all__ = ('AbsolutePathParamType', 'FileOrUrl', 'PathOrUrl')
+
 URL_TIMEOUT_SECONDS = 10
 
 
-def _check_timeout_seconds(timeout_seconds):
+def check_timeout_seconds(timeout_seconds):
     """Raise if timeout is not within range [0;60]"""
     try:
         timeout_seconds = int(timeout_seconds)
@@ -32,9 +34,7 @@ def _check_timeout_seconds(timeout_seconds):
 
 
 class AbsolutePathParamType(click.Path):
-    """
-    The ParamType for identifying absolute Paths (derived from click.Path).
-    """
+    """The ParamType for identifying absolute Paths (derived from click.Path)."""
 
     name = 'AbsolutePath'
 
@@ -50,9 +50,7 @@ class AbsolutePathParamType(click.Path):
 
 
 class AbsolutePathOrEmptyParamType(AbsolutePathParamType):
-    """
-    The ParamType for identifying absolute Paths, accepting also empty paths.
-    """
+    """The ParamType for identifying absolute Paths, accepting also empty paths."""
 
     name = 'AbsolutePathEmpty'
 
@@ -74,37 +72,27 @@ class PathOrUrl(click.Path):
         Must be an integer in the range [0;60].
     """
 
-    # pylint: disable=protected-access
-
     name = 'PathOrUrl'
 
     def __init__(self, timeout_seconds=URL_TIMEOUT_SECONDS, **kwargs):
         super().__init__(**kwargs)
 
-        self.timeout_seconds = _check_timeout_seconds(timeout_seconds)
+        self.timeout_seconds = check_timeout_seconds(timeout_seconds)
 
     def convert(self, value, param, ctx):
-        """Overwrite `convert`
-        Check first if `click.Path`-type, then check if URL.
-        """
+        """Overwrite `convert` Check first if `click.Path`-type, then check if URL."""
         try:
-            # Check if `click.Path`-type
             return super().convert(value, param, ctx)
         except click.exceptions.BadParameter:
-            # Check if URL
             return self.checks_url(value, param, ctx)
 
     def checks_url(self, url, param, ctx):
         """Check whether URL is reachable within timeout."""
         try:
-            urllib.request.urlopen(url, timeout=self.timeout_seconds)
+            with urllib.request.urlopen(url, timeout=self.timeout_seconds):
+                pass
         except (urllib.error.URLError, urllib.error.HTTPError, timeout):
-            self.fail(
-                '{0} "{1}" could not be reached within {2} s.\n'
-                'Is it a valid {3} or URL?'.format(
-                    self.path_type, click._compat.filename_to_ui(url), self.timeout_seconds, self.name
-                ), param, ctx
-            )
+            self.fail(f'{self.name} "{url}" could not be reached within {self.timeout_seconds} s.\n', param, ctx)
 
         return url
 
@@ -120,31 +108,22 @@ class FileOrUrl(click.File):
 
     name = 'FileOrUrl'
 
-    # pylint: disable=protected-access
-
     def __init__(self, timeout_seconds=URL_TIMEOUT_SECONDS, **kwargs):
         super().__init__(**kwargs)
 
-        self.timeout_seconds = _check_timeout_seconds(timeout_seconds)
+        self.timeout_seconds = check_timeout_seconds(timeout_seconds)
 
     def convert(self, value, param, ctx):
-        """Return file handle.
-        """
+        """Return file handle."""
         try:
-            # Check if `click.File`-type
             return super().convert(value, param, ctx)
         except click.exceptions.BadParameter:
-            # Check if URL
             handle = self.get_url(value, param, ctx)
         return handle
 
     def get_url(self, url, param, ctx):
         """Retrieve file from URL."""
         try:
-            return urllib.request.urlopen(url, timeout=self.timeout_seconds)
+            return urllib.request.urlopen(url, timeout=self.timeout_seconds)  # pylint: disable=consider-using-with
         except (urllib.error.URLError, urllib.error.HTTPError, timeout):
-            self.fail(
-                '"{0}" could not be reached within {1} s.\n'
-                'Is it a valid {2} or URL?.'.format(click._compat.filename_to_ui(url), self.timeout_seconds, self.name),
-                param, ctx
-            )
+            self.fail(f'{self.name} "{url}" could not be reached within {self.timeout_seconds} s.\n', param, ctx)
