@@ -8,14 +8,32 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Common utility functions for command line commands."""
-# pylint: disable=import-error
-
+import logging
 import os
 import sys
+from typing import TYPE_CHECKING
 
-import click
 from tabulate import tabulate
 
+from . import echo
+
+if TYPE_CHECKING:
+    from aiida.orm import WorkChainNode
+
+__all__ = ('is_verbose',)
+
+
+def is_verbose():
+    """Return whether the configured logging verbosity is considered verbose, i.e., equal or lower to ``INFO`` level.
+
+    .. note:: This checks the effective logging level that is set on the ``CMDLINE_LOGGER``. This means that it will
+        consider the logging level set on the parent ``AIIDA_LOGGER`` if not explicitly set on itself. The level of the
+        main logger can be manipulated from the command line through the ``VERBOSITY`` option that is available for all
+        commands.
+
+    """
+    return echo.CMDLINE_LOGGER.getEffectiveLevel() <= logging.INFO
+
 
 def get_env_with_venv_bin():
     """Create a clone of the current running environment with the AIIDA_PATH variable set directory of the config."""
@@ -49,31 +67,24 @@ def format_local_time(timestamp, format_str='%Y-%m-%d %H:%M:%S'):
 def print_last_process_state_change(process_type=None):
     """
     Print the last time that a process of the specified type has changed its state.
-    This function will also print a warning if the daemon is not running.
 
     :param process_type: optional process type for which to get the latest state change timestamp.
         Valid process types are either 'calculation' or 'work'.
     """
-    from aiida.cmdline.utils.echo import echo_info, echo_warning
+    from aiida.cmdline.utils.echo import echo_report
     from aiida.common import timezone
     from aiida.common.utils import str_timedelta
-    from aiida.engine.daemon.client import get_daemon_client
     from aiida.engine.utils import get_process_state_change_timestamp
 
-    client = get_daemon_client()
-
     timestamp = get_process_state_change_timestamp(process_type)
 
     if timestamp is None:
-        echo_info('last time an entry changed state: never')
+        echo_report('last time an entry changed state: never')
     else:
-        timedelta = timezone.delta(timestamp, timezone.now())
+        timedelta = timezone.delta(timestamp)
         formatted = format_local_time(timestamp, format_str='at %H:%M:%S on %Y-%m-%d')
         relative = str_timedelta(timedelta, negative_to_zero=True, max_num_fields=1)
-        echo_info(f'last time an entry changed state: {relative} ({formatted})')
-
-    if not client.is_daemon_running:
-        echo_warning('the daemon is not running', bold=True)
+        echo_report(f'last time an entry changed state: {relative} ({formatted})')
 
 
 def get_node_summary(node):
@@ -83,6 +94,7 @@ def get_node_summary(node):
     :return: a string summary of the node
     """
     from plumpy import ProcessState
+
     from aiida.orm import ProcessNode
 
     table_headers = ['Property', 'Value']
@@ -134,18 +146,18 @@ def get_node_info(node, include_summary=True):
     :param include_summary: boolean, if True, also include a summary of node properties
     :return: a string summary of the node including a description of all its links and log messages
     """
-    from aiida.common.links import LinkType
     from aiida import orm
+    from aiida.common.links import LinkType
 
     if include_summary:
         result = get_node_summary(node)
     else:
         result = ''
 
-    nodes_caller = node.get_incoming(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK))
-    nodes_called = node.get_outgoing(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK))
-    nodes_input = node.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
-    nodes_output = node.get_outgoing(link_type=(LinkType.CREATE, LinkType.RETURN))
+    nodes_caller = node.base.links.get_incoming(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK))
+    nodes_called = node.base.links.get_outgoing(link_type=(LinkType.CALL_CALC, LinkType.CALL_WORK))
+    nodes_input = node.base.links.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
+    nodes_output = node.base.links.get_outgoing(link_type=(LinkType.CREATE, LinkType.RETURN))
 
     if nodes_input:
         result += f"\n{format_nested_links(nodes_input.nested(), headers=['Inputs', 'PK', 'Type'])}"
@@ -161,7 +173,7 @@ def get_node_info(node, include_summary=True):
         links = sorted(nodes_called.all(), key=lambda x: x.node.ctime)
         result += f"\n{format_flat_links(links, headers=['Called', 'PK', 'Type'])}"
 
-    log_messages = orm.Log.objects.get_logs_for(node)
+    log_messages = orm.Log.collection.get_logs_for(node)
 
     if log_messages:
         table = []
@@ -183,7 +195,10 @@ def format_flat_links(links, headers):
     table = []
 
     for link_triple in links:
-        table.append([link_triple.link_label, link_triple.node.pk, link_triple.node.get_attribute('process_label', '')])
+        table.append([
+            link_triple.link_label, link_triple.node.pk,
+            link_triple.node.base.attributes.get('process_label', '')
+        ])
 
     result = f'\n{tabulate(table, headers=headers)}'
 
@@ -198,6 +213,7 @@ def format_nested_links(links, headers):
     :return: nested formatted string
     """
     from collections.abc import Mapping
+
     import tabulate as tb
 
     tb.PRESERVE_WHITESPACE = True
@@ -236,7 +252,7 @@ def get_calcjob_report(calcjob):
     from aiida import orm
     from aiida.common.datastructures import CalcJobState
 
-    log_messages = orm.Log.objects.get_logs_for(calcjob)
+    log_messages = orm.Log.collection.get_logs_for(calcjob)
     scheduler_out = calcjob.get_scheduler_stdout()
     scheduler_err = calcjob.get_scheduler_stderr()
     calcjob_state = calcjob.get_state()
@@ -291,13 +307,13 @@ def get_process_function_report(node):
 
     report = []
 
-    for log in orm.Log.objects.get_logs_for(node):
-        report.append(f'{log.time:%Y-%m-%d %H:%M:%S} [{log.id}]: {log.message}')
+    for log in orm.Log.collection.get_logs_for(node):
+        report.append(f'{log.time:%Y-%m-%d %H:%M:%S} [{log.pk}]: {log.message}')
 
     return '\n'.join(report)
 
 
-def get_workchain_report(node, levelname, indent_size=4, max_depth=None):
+def get_workchain_report(node: 'WorkChainNode', levelname, indent_size=4, max_depth=None):
     """
     Return a multi line string representation of the log messages and output of a given workchain
 
@@ -306,15 +322,16 @@ def get_workchain_report(node, levelname, indent_size=4, max_depth=None):
     """
     # pylint: disable=too-many-locals
     import itertools
+
     from aiida import orm
     from aiida.common.log import LOG_LEVELS
 
     def get_report_messages(uuid, depth, levelname):
         """Return list of log messages with given levelname and their depth for a node with a given uuid."""
-        node_id = orm.load_node(uuid).id
+        node_id = orm.load_node(uuid).pk
         filters = {'dbnode_id': node_id}
 
-        entries = orm.Log.objects.find(filters)
+        entries = orm.Log.collection.find(filters)
         entries = [entry for entry in entries if LOG_LEVELS[entry.levelname] >= LOG_LEVELS[levelname]]
         return [(_, depth) for _ in entries]
 
@@ -323,7 +340,7 @@ def get_workchain_report(node, levelname, indent_size=4, max_depth=None):
         Get a nested tree of work calculation nodes and their nesting level starting from this uuid.
         The result is a list of uuid of these nodes.
         """
-        builder = orm.QueryBuilder()
+        builder = orm.QueryBuilder(backend=node.backend)
         builder.append(cls=orm.WorkChainNode, filters={'uuid': uuid}, tag='workcalculation')
         builder.append(
             cls=orm.WorkChainNode,
@@ -356,7 +373,7 @@ def get_workchain_report(node, levelname, indent_size=4, max_depth=None):
     if not reports:
         return 'No log messages recorded for this entry'
 
-    log_ids = [entry[0].id for entry in reports]
+    log_ids = [entry[0].pk for entry in reports]
     levelnames = [len(entry[0].levelname) for entry in reports]
     width_id = len(str(max(log_ids)))
     width_levelname = max(levelnames)
@@ -364,7 +381,7 @@ def get_workchain_report(node, levelname, indent_size=4, max_depth=None):
 
     for entry, depth in reports:
         line = '{time:%Y-%m-%d %H:%M:%S} [{id:<{width_id}} | {levelname:>{width_levelname}}]:{indent} {message}'.format(
-            id=entry.id,
+            id=entry.pk,
             levelname=entry.levelname,
             message=entry.message,
             time=entry.time,
@@ -390,10 +407,10 @@ def print_process_info(process):
     if not docstring:
         docstring = ['No description available']
 
-    click.secho('Description:\n', fg='red', bold=True)
+    echo.echo('Description:\n', fg='red', bold=True)
     for line in docstring:
-        click.echo(f'	{line.lstrip()}')
-    click.echo()
+        echo.echo(f'    {line.lstrip()}')
+    echo.echo('')
 
     print_process_spec(process.spec())
 
@@ -433,26 +450,26 @@ def print_process_spec(process_spec):
     max_width_type = max([len(entry[2]) for entry in inputs + outputs]) + 2
 
     if process_spec.inputs:
-        click.secho('Inputs:', fg='red', bold=True)
+        echo.echo('Inputs:', fg='red', bold=True)
     for entry in inputs:
         if entry[1] == 'required':
-            click.secho(template.format(*entry, width_name=max_width_name, width_type=max_width_type), bold=True)
+            echo.echo(template.format(*entry, width_name=max_width_name, width_type=max_width_type), bold=True)
         else:
-            click.secho(template.format(*entry, width_name=max_width_name, width_type=max_width_type))
+            echo.echo(template.format(*entry, width_name=max_width_name, width_type=max_width_type))
 
     if process_spec.outputs:
-        click.secho('Outputs:', fg='red', bold=True)
+        echo.echo('Outputs:', fg='red', bold=True)
     for entry in outputs:
         if entry[1] == 'required':
-            click.secho(template.format(*entry, width_name=max_width_name, width_type=max_width_type), bold=True)
+            echo.echo(template.format(*entry, width_name=max_width_name, width_type=max_width_type), bold=True)
         else:
-            click.secho(template.format(*entry, width_name=max_width_name, width_type=max_width_type))
+            echo.echo(template.format(*entry, width_name=max_width_name, width_type=max_width_type))
 
     if process_spec.exit_codes:
-        click.secho('Exit codes:', fg='red', bold=True)
+        echo.echo('Exit codes:', fg='red', bold=True)
     for exit_code in sorted(process_spec.exit_codes.values(), key=lambda exit_code: exit_code.status):
-        message = exit_code.message.capitalize()
-        click.secho('{:>{width_name}d}:  {}'.format(exit_code.status, message, width_name=max_width_name))
+        message = exit_code.message
+        echo.echo('{:>{width_name}d}:  {}'.format(exit_code.status, message, width_name=max_width_name))
 
 
 def get_num_workers():
@@ -460,7 +477,7 @@ def get_num_workers():
     Get the number of active daemon workers from the circus client
     """
     from aiida.common.exceptions import CircusCallError
-    from aiida.manage.manager import get_manager
+    from aiida.manage import get_manager
 
     manager = get_manager()
     client = manager.get_daemon_client()
@@ -476,38 +493,41 @@ def get_num_workers():
                 raise CircusCallError
         try:
             return response['numprocesses']
-        except KeyError:
-            raise CircusCallError('Circus did not return the number of daemon processes')
+        except KeyError as exc:
+            raise CircusCallError('Circus did not return the number of daemon processes') from exc
 
 
 def check_worker_load(active_slots):
-    """
-    Check if the percentage usage of the daemon worker slots exceeds a threshold.
-    If it does, print a warning.
+    """Log a message with information on the current daemon worker load.
 
-    The purpose of this check is to warn the user if they are close to running out of worker slots
-    which could lead to their processes becoming stuck indefinitely.
+    If there are daemon workers active, it logs the current load. If that exceeds 90%, a warning is included with the
+    suggestion to run ``verdi daemon incr``.
+
+    The purpose of this check is to warn the user if they are close to running out of worker slots which could lead to
+    their processes becoming stuck indefinitely.
 
     :param active_slots: the number of currently active worker slots
     """
-    from aiida.cmdline.utils import echo
     from aiida.common.exceptions import CircusCallError
-    from aiida.manage.configuration import get_config
+    from aiida.manage import get_config_option
 
     warning_threshold = 0.9  # 90%
 
-    config = get_config()
-    slots_per_worker = config.get_option('daemon.worker_process_slots', config.current_profile.name)
+    slots_per_worker = get_config_option('daemon.worker_process_slots')
 
     try:
         active_workers = get_num_workers()
     except CircusCallError:
-        echo.echo_critical('Could not contact Circus to get the number of active workers')
+        echo.echo_critical('Could not contact Circus to get the number of active workers.')
 
     if active_workers is not None:
         available_slots = active_workers * slots_per_worker
         percent_load = 1.0 if not available_slots else (active_slots / available_slots)
         if percent_load > warning_threshold:
             echo.echo('')  # New line
-            echo.echo_warning(f'{percent_load * 100:.0f}% of the available daemon worker slots have been used!')
-            echo.echo_warning("Increase the number of workers with 'verdi daemon incr'.\n")
+            echo.echo_warning(f'{percent_load * 100:.0f}%% of the available daemon worker slots have been used!')
+            echo.echo_warning('Increase the number of workers with `verdi daemon incr`.')
+        else:
+            echo.echo_report(f'Using {percent_load * 100:.0f}%% of the available daemon worker slots.')
+    else:
+        echo.echo_report('No active daemon workers.')
