@@ -8,8 +8,8 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """A utility module with a factory of standard QueryBuilder instances for Calculation nodes."""
-from aiida.common.lang import classproperty
 from aiida.cmdline.utils.query.mapping import CalculationProjectionMapper
+from aiida.common.lang import classproperty
 
 
 class CalculationQueryBuilder:
@@ -138,7 +138,7 @@ class CalculationQueryBuilder:
 
         if relationships is not None:
             for tag, entity in relationships.items():
-                builder.append(cls=type(entity), filters={'id': entity.id}, **{tag: 'process'})
+                builder.append(cls=type(entity), filters={'id': entity.pk}, **{tag: 'process'})
 
         if order_by is not None:
             builder.order_by({'process': order_by})
