@@ -17,10 +17,10 @@ def format_relative_time(datetime):
     :param datetime: the datetime to format
     :return: string representation of the relative time since the given datetime
     """
-    from aiida.common.utils import str_timedelta
     from aiida.common import timezone
+    from aiida.common.utils import str_timedelta
 
-    timedelta = timezone.delta(datetime, timezone.now())
+    timedelta = timezone.delta(datetime)
 
     return str_timedelta(timedelta, negative_to_zero=True, max_num_fields=1)
 
