@@ -11,7 +11,8 @@
 from functools import partial
 from inspect import getfullargspec
 from types import FunctionType  # pylint: disable=no-name-in-module
-from typing import List, Optional, Union, NamedTuple
+from typing import List, NamedTuple, Optional, Union
+
 from wrapt import decorator
 
 from ..exit_code import ExitCode
@@ -90,7 +91,7 @@ def process_handler(
     if exit_codes is not None and not isinstance(exit_codes, list):
         exit_codes = [exit_codes]
 
-    if exit_codes and any([not isinstance(exit_code, ExitCode) for exit_code in exit_codes]):
+    if exit_codes and any(not isinstance(exit_code, ExitCode) for exit_code in exit_codes):
         raise TypeError('`exit_codes` keyword should be an instance of `ExitCode` or list thereof.')
 
     if not isinstance(enabled, bool):
@@ -120,7 +121,7 @@ def process_handler(
 
         # Append the name and return value of the current process handler to the `considered_handlers` extra.
         try:
-            considered_handlers = instance.node.get_extra(instance._considered_handlers_extra, [])  # pylint: disable=protected-access
+            considered_handlers = instance.node.base.extras.get(instance._considered_handlers_extra, [])  # pylint: disable=protected-access
             current_process = considered_handlers[-1]
         except IndexError:
             # The extra was never initialized, so we skip this functionality
@@ -131,7 +132,7 @@ def process_handler(
             if isinstance(serialized, ProcessHandlerReport):
                 serialized = {'do_break': serialized.do_break, 'exit_status': serialized.exit_code.status}
             current_process.append((wrapped.__name__, serialized))
-            instance.node.set_extra(instance._considered_handlers_extra, considered_handlers)  # pylint: disable=protected-access
+            instance.node.base.extras.set(instance._considered_handlers_extra, considered_handlers)  # pylint: disable=protected-access
 
         return result
 
