@@ -11,11 +11,13 @@
 import collections.abc
 import functools
 import logging
-from typing import Any, List, Optional, Sequence, Union, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, List, Optional, Sequence, Tuple, Union
 
 from plumpy.persistence import auto_persist
-from plumpy.process_states import Wait, Continue
-from plumpy.workchains import if_, while_, return_, _PropagateReturn, Stepper, WorkChainSpec as PlumpyWorkChainSpec
+from plumpy.process_states import Continue, Wait
+from plumpy.workchains import Stepper
+from plumpy.workchains import WorkChainSpec as PlumpyWorkChainSpec
+from plumpy.workchains import _PropagateReturn, if_, return_, while_
 
 from aiida.common import exceptions
 from aiida.common.extendeddicts import AttributeDict
@@ -24,9 +26,9 @@ from aiida.orm import Node, ProcessNode, WorkChainNode
 from aiida.orm.utils import load_node
 
 from ..exit_code import ExitCode
-from ..process_spec import ProcessSpec
 from ..process import Process, ProcessState
-from .awaitable import Awaitable, AwaitableTarget, AwaitableAction, construct_awaitable
+from ..process_spec import ProcessSpec
+from .awaitable import Awaitable, AwaitableAction, AwaitableTarget, construct_awaitable
 
 if TYPE_CHECKING:
     from aiida.engine.runners import Runner
@@ -121,25 +123,59 @@ class WorkChain(Process):
         super().on_run()
         self.node.set_stepper_state_info(str(self._stepper))
 
+    def _resolve_nested_context(self, key: str) -> Tuple[AttributeDict, str]:
+        """
+        Returns a reference to a sub-dictionary of the context and the last key,
+        after resolving a potentially segmented key where required sub-dictionaries are created as needed.
+
+        :param key: A key into the context, where words before a dot are interpreted as a key for a sub-dictionary
+        """
+        ctx = self.ctx
+        ctx_path = key.split('.')
+
+        for index, path in enumerate(ctx_path[:-1]):
+            try:
+                ctx = ctx[path]
+            except KeyError:  # see below why this is the only exception we have to catch here
+                ctx[path] = AttributeDict()  # create the sub-dict and update the context
+                ctx = ctx[path]
+                continue
+
+            # Notes:
+            # * the first ctx (self.ctx) is guaranteed to be an AttributeDict, hence the post-"dereference" checking
+            # * the values can be many different things: on insertion they are either AtrributeDict, List or Awaitables
+            #   (subclasses of AttributeDict) but after resolution of an Awaitable this will be the value itself
+            # * assumption: a resolved value is never a plain AttributeDict, on the other hand if a resolved Awaitable
+            #   would be an AttributeDict we can append things to it since the order of tasks is maintained.
+            if type(ctx) != AttributeDict:  # pylint: disable=C0123
+                raise ValueError(
+                    f'Can not update the context for key `{key}`:'
+                    f' found instance of `{type(ctx)}` at `{".".join(ctx_path[:index+1])}`, expected AttributeDict'
+                )
+
+        return ctx, ctx_path[-1]
+
     def insert_awaitable(self, awaitable: Awaitable) -> None:
         """Insert an awaitable that should be terminated before before continuing to the next step.
 
         :param awaitable: the thing to await
-        :type awaitable: :class:`aiida.engine.processes.workchains.awaitable.Awaitable`
         """
-        self._awaitables.append(awaitable)
+        ctx, key = self._resolve_nested_context(awaitable.key)
 
         # Already assign the awaitable itself to the location in the context container where it is supposed to end up
         # once it is resolved. This is especially important for the `APPEND` action, since it needs to maintain the
         # order, but the awaitables will not necessarily be resolved in the order in which they are added. By using the
         # awaitable as a placeholder, in the `resolve_awaitable`, it can be found and replaced by the resolved value.
         if awaitable.action == AwaitableAction.ASSIGN:
-            self.ctx[awaitable.key] = awaitable
+            ctx[key] = awaitable
         elif awaitable.action == AwaitableAction.APPEND:
-            self.ctx.setdefault(awaitable.key, []).append(awaitable)
+            ctx.setdefault(key, []).append(awaitable)
         else:
-            assert f'Unknown awaitable action: {awaitable.action}'
+            raise AssertionError(f'Unsupported awaitable action: {awaitable.action}')
 
+        self._awaitables.append(
+            awaitable
+        )  # add only if everything went ok, otherwise we end up in an inconsistent state
         self._update_process_status()
 
     def resolve_awaitable(self, awaitable: Awaitable, value: Any) -> None:
@@ -149,23 +185,25 @@ class WorkChain(Process):
 
         :param awaitable: the awaitable to resolve
         """
-        self._awaitables.remove(awaitable)
+
+        ctx, key = self._resolve_nested_context(awaitable.key)
 
         if awaitable.action == AwaitableAction.ASSIGN:
-            self.ctx[awaitable.key] = value
+            ctx[key] = value
         elif awaitable.action == AwaitableAction.APPEND:
             # Find the same awaitable inserted in the context
-            container = self.ctx[awaitable.key]
+            container = ctx[key]
             for index, placeholder in enumerate(container):
-                if placeholder.pk == awaitable.pk and isinstance(placeholder, Awaitable):
+                if isinstance(placeholder, Awaitable) and placeholder.pk == awaitable.pk:
                     container[index] = value
                     break
             else:
-                assert f'Awaitable `{awaitable.pk} was not found in `ctx.{awaitable.pk}`'
+                raise AssertionError(f'Awaitable `{awaitable.pk} was not found in `ctx.{awaitable.key}`')
         else:
-            assert f'Unknown awaitable action: {awaitable.action}'
+            raise AssertionError(f'Unsupported awaitable action: {awaitable.action}')
 
         awaitable.resolved = True
+        self._awaitables.remove(awaitable)  # remove only if everything went ok, otherwise we may lose track
 
         if not self.has_terminated():
             # the process may be terminated, for example, if the process was killed or excepted
@@ -301,7 +339,7 @@ class WorkChain(Process):
             raise ValueError(f'provided pk<{awaitable.pk}> could not be resolved to a valid Node instance')
 
         if awaitable.outputs:
-            value = {entry.link_label: entry.node for entry in node.get_outgoing()}
+            value = {entry.link_label: entry.node for entry in node.base.links.get_outgoing()}
         else:
             value = node
 
