@@ -9,22 +9,27 @@
 ###########################################################################
 """AiiDA Group entites"""
 from abc import ABCMeta
-from enum import Enum
+from functools import cached_property
+from typing import TYPE_CHECKING, Any, ClassVar, Optional, Sequence, Tuple, Type, TypeVar, Union, cast
 import warnings
 
 from aiida.common import exceptions
-from aiida.common.lang import type_check
-from aiida.common.warnings import AiidaDeprecationWarning
-from aiida.manage.manager import get_manager
+from aiida.common.lang import classproperty, type_check
+from aiida.common.warnings import warn_deprecation
+from aiida.manage import get_manager
 
-from . import convert
-from . import entities
-from . import users
+from . import convert, entities, extras, users
 
-__all__ = ('Group', 'GroupTypeString', 'AutoGroup', 'ImportGroup', 'UpfFamily')
+if TYPE_CHECKING:
+    from aiida.orm import Node, User
+    from aiida.orm.implementation import BackendGroup, StorageBackend
+
+__all__ = ('Group', 'AutoGroup', 'ImportGroup', 'UpfFamily')
+
+SelfType = TypeVar('SelfType', bound='Group')
 
 
-def load_group_class(type_string):
+def load_group_class(type_string: str) -> Type['Group']:
     """Load the sub class of `Group` that corresponds to the given `type_string`.
 
     .. note:: will fall back on `aiida.orm.groups.Group` if `type_string` cannot be resolved to loadable entry point.
@@ -53,124 +58,127 @@ class GroupMeta(ABCMeta):
 
         newcls = ABCMeta.__new__(cls, name, bases, namespace, **kwargs)  # pylint: disable=too-many-function-args
 
-        entry_point_group, entry_point = get_entry_point_from_class(namespace['__module__'], name)
+        mod = namespace['__module__']
+        entry_point_group, entry_point = get_entry_point_from_class(mod, name)
 
         if entry_point_group is None or entry_point_group != 'aiida.groups':
-            newcls._type_string = None
-            message = f'no registered entry point for `{name}` so its instances will not be storable.'
+            newcls._type_string = None  # type: ignore[attr-defined]
+            message = f'no registered entry point for `{mod}:{name}` so its instances will not be storable.'
             warnings.warn(message)  # pylint: disable=no-member
         else:
-            newcls._type_string = entry_point.name  # pylint: disable=protected-access
+            assert entry_point is not None
+            newcls._type_string = cast(str, entry_point.name)  # type: ignore[attr-defined]  # pylint: disable=protected-access
 
         return newcls
 
 
-class GroupTypeString(Enum):
-    """A simple enum of allowed group type strings.
+class GroupCollection(entities.Collection['Group']):
+    """Collection of Groups"""
 
-    .. deprecated:: 1.2.0
-        This enum is deprecated and will be removed in `v2.0.0`.
-    """
-    UPFGROUP_TYPE = 'data.upf'
-    IMPORTGROUP_TYPE = 'auto.import'
-    VERDIAUTOGROUP_TYPE = 'auto.run'
-    USER = 'user'
+    @staticmethod
+    def _entity_base_cls() -> Type['Group']:
+        return Group
+
+    def get_or_create(self, label: Optional[str] = None, **kwargs) -> Tuple['Group', bool]:
+        """
+        Try to retrieve a group from the DB with the given arguments;
+        create (and store) a new group if such a group was not present yet.
+
+        :param label: group label
+
+        :return: (group, created) where group is the group (new or existing,
+            in any case already stored) and created is a boolean saying
+        """
+        if not label:
+            raise ValueError('Group label must be provided')
+
+        res = self.find(filters={'label': label})
+
+        if not res:
+            return self.entity_type(label, backend=self.backend, **kwargs).store(), True
+
+        if len(res) > 1:
+            raise exceptions.MultipleObjectsError('More than one groups found in the database')
+
+        return res[0], False
+
+    def delete(self, pk: int) -> None:
+        """
+        Delete a group
+
+        :param pk: the id of the group to delete
+        """
+        self._backend.groups.delete(pk)
 
 
-class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
+class GroupBase:
+    """A namespace for group related functionality, that is not directly related to its user-facing properties."""
+
+    def __init__(self, group: 'Group') -> None:
+        """Construct a new instance of the base namespace."""
+        self._group: 'Group' = group
+
+    @cached_property
+    def extras(self) -> extras.EntityExtras:
+        """Return the extras of this group."""
+        return extras.EntityExtras(self._group)
+
+
+class Group(entities.Entity['BackendGroup'], metaclass=GroupMeta):
     """An AiiDA ORM implementation of group of nodes."""
 
-    class Collection(entities.Collection):
-        """Collection of Groups"""
+    # added by metaclass
+    _type_string: ClassVar[Optional[str]]
 
-        def get_or_create(self, label=None, **kwargs):
-            """
-            Try to retrieve a group from the DB with the given arguments;
-            create (and store) a new group if such a group was not present yet.
+    _CLS_COLLECTION = GroupCollection
 
-            :param label: group label
-            :type label: str
-
-            :return: (group, created) where group is the group (new or existing,
-              in any case already stored) and created is a boolean saying
-            :rtype: (:class:`aiida.orm.Group`, bool)
-            """
-            if not label:
-                raise ValueError('Group label must be provided')
-
-            res = self.find(filters={'label': label})
-
-            if not res:
-                return self.entity_type(label, backend=self.backend, **kwargs).store(), True
-
-            if len(res) > 1:
-                raise exceptions.MultipleObjectsError('More than one groups found in the database')
-
-            return res[0], False
-
-        def delete(self, id):  # pylint: disable=invalid-name, redefined-builtin
-            """
-            Delete a group
-
-            :param id: the id of the group to delete
-            """
-            self._backend.groups.delete(id)
-
-    def __init__(self, label=None, user=None, description='', type_string=None, backend=None):
+    def __init__(
+        self,
+        label: Optional[str] = None,
+        user: Optional['User'] = None,
+        description: str = '',
+        type_string: Optional[str] = None,
+        backend: Optional['StorageBackend'] = None
+    ):
         """
         Create a new group. Either pass a dbgroup parameter, to reload
         a group from the DB (and then, no further parameters are allowed),
         or pass the parameters for the Group creation.
 
-        .. deprecated:: 1.2.0
-            The parameter `type_string` will be removed in `v2.0.0` and is now determined automatically.
-
         :param label: The group label, required on creation
-        :type label: str
-
         :param description: The group description (by default, an empty string)
-        :type description: str
-
         :param user: The owner of the group (by default, the automatic user)
-        :type user: :class:`aiida.orm.User`
-
         :param type_string: a string identifying the type of group (by default,
             an empty string, indicating an user-defined group.
-        :type type_string: str
         """
         if not label:
             raise ValueError('Group label must be provided')
 
-        if type_string is not None:
-            message = '`type_string` is deprecated because it is determined automatically'
-            warnings.warn(message)  # pylint: disable=no-member
-
-        # If `type_string` is explicitly defined, override automatically determined `self._type_string`. This is
-        # necessary for backwards compatibility.
-        if type_string is not None:
-            self._type_string = type_string
-
-        type_string = self._type_string
-
-        backend = backend or get_manager().get_backend()
-        user = user or users.User.objects(backend).get_default()
+        backend = backend or get_manager().get_profile_storage()
+        user = user or backend.default_user
         type_check(user, users.User)
+        type_string = self._type_string
 
         model = backend.groups.create(
             label=label, user=user.backend_entity, description=description, type_string=type_string
         )
         super().__init__(model)
 
-    def __repr__(self):
-        return f'<{self.__class__.__name__}: {str(self)}>'
+    @cached_property
+    def base(self) -> GroupBase:
+        """Return the group base namespace."""
+        return GroupBase(self)
 
-    def __str__(self):
-        if self.type_string:
-            return f'"{self.label}" [type {self.type_string}], of user {self.user.email}'
+    def __repr__(self) -> str:
+        return (
+            f'<{self.__class__.__name__}: {self.label!r} '
+            f'[{"type " + self.type_string if self.type_string else "user-defined"}], of user {self.user.email}>'
+        )
 
-        return f'"{self.label}" [user-defined], of user {self.user.email}'
+    def __str__(self) -> str:
+        return f'{self.__class__.__name__}<{self.label}>'
 
-    def store(self):
+    def store(self: SelfType) -> SelfType:
         """Verify that the group is allowed to be stored, which is the case along as `type_string` is set."""
         if self._type_string is None:
             raise exceptions.StoringNotAllowed('`type_string` is `None` so the group cannot be stored.')
@@ -178,14 +186,24 @@ class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
         return super().store()
 
     @property
-    def label(self):
+    def uuid(self) -> str:
+        """Return the UUID for this group.
+
+        This identifier is unique across all entities types and backend instances.
+
+        :return: the entity uuid
+        """
+        return self._backend_entity.uuid
+
+    @property
+    def label(self) -> str:
         """
         :return: the label of the group as a string
         """
         return self._backend_entity.label
 
     @label.setter
-    def label(self, label):
+    def label(self, label: str) -> None:
         """
         Attempt to change the label of the group instance. If the group is already stored
         and the another group of the same type already exists with the desired label, a
@@ -199,79 +217,63 @@ class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
         self._backend_entity.label = label
 
     @property
-    def description(self):
+    def description(self) -> str:
         """
         :return: the description of the group as a string
-        :rtype: str
         """
         return self._backend_entity.description
 
     @description.setter
-    def description(self, description):
+    def description(self, description: str) -> None:
         """
         :param description: the description of the group as a string
-        :type description: str
-
         """
         self._backend_entity.description = description
 
     @property
-    def type_string(self):
+    def type_string(self) -> str:
         """
         :return: the string defining the type of the group
         """
         return self._backend_entity.type_string
 
     @property
-    def user(self):
+    def user(self) -> 'User':
         """
         :return: the user associated with this group
         """
         return users.User.from_backend_entity(self._backend_entity.user)
 
     @user.setter
-    def user(self, user):
+    def user(self, user: 'User') -> None:
         """Set the user.
 
         :param user: the user
-        :type user: :class:`aiida.orm.User`
         """
         type_check(user, users.User)
         self._backend_entity.user = user.backend_entity
 
-    @property
-    def uuid(self):
-        """
-        :return: a string with the uuid
-        :rtype: str
-        """
-        return self._backend_entity.uuid
-
-    def count(self):
+    def count(self) -> int:
         """Return the number of entities in this group.
 
         :return: integer number of entities contained within the group
-        :rtype: int
         """
         return self._backend_entity.count()
 
     @property
-    def nodes(self):
+    def nodes(self) -> convert.ConvertIterator:
         """
         Return a generator/iterator that iterates over all nodes and returns
         the respective AiiDA subclasses of Node, and also allows to ask for
         the number of nodes in the group using len().
-
-        :rtype: :class:`aiida.orm.convert.ConvertIterator`
         """
         return convert.ConvertIterator(self._backend_entity.nodes)
 
     @property
-    def is_empty(self):
+    def is_empty(self) -> bool:
         """Return whether the group is empty, i.e. it does not contain any nodes.
 
         :return: True if it contains no nodes, False otherwise
-        :rtype: bool
         """
         try:
             self.nodes[0]
@@ -280,17 +282,16 @@ class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
         else:
             return False
 
-    def clear(self):
+    def clear(self) -> None:
         """Remove all the nodes from this group."""
         return self._backend_entity.clear()
 
-    def add_nodes(self, nodes):
+    def add_nodes(self, nodes: Union['Node', Sequence['Node']]) -> None:
         """Add a node or a set of nodes to the group.
 
         :note: all the nodes *and* the group itself have to be stored.
 
         :param nodes: a single `Node` or a list of `Nodes`
-        :type nodes: :class:`aiida.orm.Node` or list
         """
         from .nodes import Node
 
@@ -306,13 +307,12 @@ class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
 
         self._backend_entity.add_nodes([node.backend_entity for node in nodes])
 
-    def remove_nodes(self, nodes):
+    def remove_nodes(self, nodes: Union['Node', Sequence['Node']]) -> None:
         """Remove a node or a set of nodes to the group.
 
         :note: all the nodes *and* the group itself have to be stored.
 
         :param nodes: a single `Node` or a list of `Nodes`
-        :type nodes: :class:`aiida.orm.Node` or list
         """
         from .nodes import Node
 
@@ -328,98 +328,45 @@ class Group(entities.Entity, entities.EntityExtrasMixin, metaclass=GroupMeta):
 
         self._backend_entity.remove_nodes([node.backend_entity for node in nodes])
 
-    @classmethod
-    def get(cls, **kwargs):
-        """
-        Custom get for group which can be used to get a group with the given attributes
-
-        :param kwargs: the attributes to match the group to
-
-        :return: the group
-        :type nodes: :class:`aiida.orm.Node` or list
-        """
-        from aiida.orm import QueryBuilder
-
-        if 'type_string' in kwargs:
-            message = '`type_string` is deprecated because it is determined automatically'
-            warnings.warn(message)  # pylint: disable=no-member
-            type_check(kwargs['type_string'], str)
-
-        return QueryBuilder().append(cls, filters=kwargs).one()[0]
-
-    def is_user_defined(self):
+    def is_user_defined(self) -> bool:
         """
         :return: True if the group is user defined, False otherwise
-        :rtype: bool
         """
         return not self.type_string
 
-    @staticmethod
-    def get_schema():
+    _deprecated_extra_methods = {
+        'extras': 'all',
+        'get_extra': 'get',
+        'get_extra_many': 'get_many',
+        'set_extra': 'set',
+        'set_extra_many': 'set_many',
+        'reset_extras': 'reset',
+        'delete_extra': 'delete',
+        'delete_extra_many': 'delete_many',
+        'clear_extras': 'clear',
+        'extras_items': 'items',
+        'extras_keys': 'keys',
+    }
+
+    def __getattr__(self, name: str) -> Any:
         """
-        Every node property contains:
-            - display_name: display name of the property
-            - help text: short help text of the property
-            - is_foreign_key: is the property foreign key to other type of the node
-            - type: type of the property. e.g. str, dict, int
-
-        :return: schema of the group
-        :rtype: dict
-
-        .. deprecated:: 1.0.0
-
-            Will be removed in `v2.0.0`.
-            Use :meth:`~aiida.restapi.translator.base.BaseTranslator.get_projectable_properties` instead.
+        This method is called when an extras is not found in the instance.
 
+        It allows for the handling of deprecated mixin methods.
         """
-        message = 'method is deprecated, use' \
-            '`aiida.restapi.translator.base.BaseTranslator.get_projectable_properties` instead'
-        warnings.warn(message, AiidaDeprecationWarning)  # pylint: disable=no-member
+        if name in self._deprecated_extra_methods:
+            new_name = self._deprecated_extra_methods[name]
+            kls = self.__class__.__name__
+            warn_deprecation(
+                f'`{kls}.{name}` is deprecated, use `{kls}.base.extras.{new_name}` instead.', version=3, stacklevel=3
+            )
+            return getattr(self.base.extras, new_name)
 
-        return {
-            'description': {
-                'display_name': 'Description',
-                'help_text': 'Short description of the group',
-                'is_foreign_key': False,
-                'type': 'str'
-            },
-            'id': {
-                'display_name': 'Id',
-                'help_text': 'Id of the object',
-                'is_foreign_key': False,
-                'type': 'int'
-            },
-            'label': {
-                'display_name': 'Label',
-                'help_text': 'Name of the object',
-                'is_foreign_key': False,
-                'type': 'str'
-            },
-            'type_string': {
-                'display_name': 'Type_string',
-                'help_text': 'Type of the group',
-                'is_foreign_key': False,
-                'type': 'str'
-            },
-            'user_id': {
-                'display_name': 'Id of creator',
-                'help_text': 'Id of the user that created the node',
-                'is_foreign_key': True,
-                'related_column': 'id',
-                'related_resource': '_dbusers',
-                'type': 'int'
-            },
-            'uuid': {
-                'display_name': 'Unique ID',
-                'help_text': 'Universally Unique Identifier',
-                'is_foreign_key': False,
-                'type': 'unicode'
-            }
-        }
+        raise AttributeError(name)
 
 
 class AutoGroup(Group):
-    """Group to be used to contain selected nodes generated while `aiida.orm.autogroup.CURRENT_AUTOGROUP` is set."""
+    """Group to be used to contain selected nodes generated, whilst autogrouping is enabled."""
 
 
 class ImportGroup(Group):
