@@ -11,10 +11,13 @@
 """Tools for handling Crystallographic Information Files (CIF)"""
 
 import re
+
 from aiida.common.utils import Capturing
 
 from .singlefile import SinglefileData
 
+__all__ = ('CifData', 'cif_from_ase', 'has_pycifrw', 'pycifrw_from_cif')
+
 ase_loops = {
     '_atom_site': [
         '_atom_site_label',
@@ -50,14 +53,14 @@ def cif_from_ase(ase, full_occupancies=False, add_fake_biso=False):
     """
     Construct a CIF datablock from the ASE structure. The code is taken
     from
-    https://wiki.fysik.dtu.dk/ase/epydoc/ase.io.cif-pysrc.html#write_cif,
+    https://wiki.fysik.dtu.dk/ase/ase/io/formatoptions.html#ase.io.cif.write_cif,
     as the original ASE code contains a bug in printing the
     Hermann-Mauguin symmetry space group symbol.
 
     :param ase: ASE "images"
     :return: array of CIF datablocks
     """
-    from numpy import arccos, pi, dot
+    from numpy import arccos, dot, pi
     from numpy.linalg import norm
 
     if not isinstance(ase, (list, tuple)):
@@ -65,7 +68,7 @@ def cif_from_ase(ase, full_occupancies=False, add_fake_biso=False):
 
     datablocks = []
     for _, atoms in enumerate(ase):
-        datablock = dict()
+        datablock = {}
 
         cell = atoms.cell
         a = norm(cell[0])
@@ -140,7 +143,7 @@ def pycifrw_from_cif(datablocks, loops=None, names=None):
         raise ImportError(f'{str(exc)}. You need to install the PyCifRW package.')
 
     if loops is None:
-        loops = dict()
+        loops = {}
 
     cif = CifFile.CifFile()  # pylint: disable=no-member
     try:
@@ -172,10 +175,7 @@ def pycifrw_from_cif(datablocks, loops=None, names=None):
                         row_size = len(tag_values)
                     elif row_size != len(tag_values):
                         raise ValueError(
-                            'Number of values for tag '
-                            "'{}' is different from "
-                            'the others in the same '
-                            'loop'.format(tag)
+                            f'Number of values for tag `{tag}` is different from the others in the same loop'
                         )
                     if row_size == 0:
                         continue
@@ -259,17 +259,7 @@ class CifData(SinglefileData):
     _values = None
     _ase = None
 
-    def __init__(
-        self,
-        ase=None,
-        file=None,
-        filename=None,
-        values=None,
-        source=None,
-        scan_type=None,
-        parse_policy=None,
-        **kwargs
-    ):
+    def __init__(self, ase=None, file=None, filename=None, values=None, scan_type=None, parse_policy=None, **kwargs):
         """Construct a new instance and set the contents to that of the file.
 
         :param file: an absolute filepath or filelike object for CIF.
@@ -277,7 +267,6 @@ class CifData(SinglefileData):
         :param filename: specify filename to use (defaults to name of provided file).
         :param ase: ASE Atoms object to construct the CifData instance from.
         :param values: PyCifRW CifFile object to construct the CifData instance from.
-        :param source:
         :param scan_type: scan type string for parsing with PyCIFRW ('standard' or 'flex'). See CifFile.ReadCif
         :param parse_policy: 'eager' (parse CIF file on set_file) or 'lazy' (defer parsing until needed)
         """
@@ -298,16 +287,13 @@ class CifData(SinglefileData):
         self.set_scan_type(scan_type or CifData._SCAN_TYPE_DEFAULT)
         self.set_parse_policy(parse_policy or CifData._PARSE_POLICY_DEFAULT)
 
-        if source is not None:
-            self.set_source(source)
-
         if ase is not None:
             self.set_ase(ase)
 
         if values is not None:
             self.set_values(values)
 
-        if not self.is_stored and file is not None and self.get_attribute('parse_policy') == 'eager':
+        if not self.is_stored and file is not None and self.base.attributes.get('parse_policy') == 'eager':
             self.parse()
 
     @staticmethod
@@ -340,7 +326,7 @@ class CifData(SinglefileData):
         return struct_list[index]
 
     @classmethod
-    def from_md5(cls, md5):
+    def from_md5(cls, md5, backend=None):
         """
         Return a list of all CIF files that match a given MD5 hash.
 
@@ -348,7 +334,7 @@ class CifData(SinglefileData):
             otherwise the CIF file will not be found.
         """
         from aiida.orm.querybuilder import QueryBuilder
-        builder = QueryBuilder()
+        builder = QueryBuilder(backend=backend)
         builder.append(cls, filters={'attributes.md5': {'==': md5}})
         return builder.all(flat=True)
 
@@ -369,6 +355,7 @@ class CifData(SinglefileData):
             from the DB.
         """
         import os
+
         from aiida.common.files import md5_file
 
         if not os.path.abspath(filename):
@@ -418,9 +405,7 @@ class CifData(SinglefileData):
         if not kwargs and self._ase:
             return self.ase
         with self.open() as handle:
-            cif = CifData.read_cif(handle, **kwargs)
-
-        return cif
+            return CifData.read_cif(handle, **kwargs)
 
     def set_ase(self, aseatoms):
         """
@@ -452,7 +437,7 @@ class CifData(SinglefileData):
             from CifFile import CifBlock  # pylint: disable=no-name-in-module
 
             with self.open() as handle:
-                c = CifFile.ReadCif(handle, scantype=self.get_attribute('scan_type', CifData._SCAN_TYPE_DEFAULT))  # pylint: disable=no-member
+                c = CifFile.ReadCif(handle, scantype=self.base.attributes.get('scan_type', CifData._SCAN_TYPE_DEFAULT))  # pylint: disable=no-member
             for k, v in c.items():
                 c.dictionary[k] = CifBlock(v)
             self._values = c
@@ -473,7 +458,8 @@ class CifData(SinglefileData):
             with Capturing():
                 tmpf.write(values.WriteOut())
             tmpf.flush()
-            self.set_file(tmpf.name)
+            tmpf.seek(0)
+            self.set_file(tmpf)
 
         self._values = values
 
@@ -491,15 +477,15 @@ class CifData(SinglefileData):
             self.set_scan_type(scan_type)
 
         # Note: this causes parsing, if not already parsed
-        self.set_attribute('formulae', self.get_formulae())
-        self.set_attribute('spacegroup_numbers', self.get_spacegroup_numbers())
+        self.base.attributes.set('formulae', self.get_formulae())
+        self.base.attributes.set('spacegroup_numbers', self.get_spacegroup_numbers())
 
     def store(self, *args, **kwargs):  # pylint: disable=signature-differs
         """
         Store the node.
         """
         if not self.is_stored:
-            self.set_attribute('md5', self.generate_md5())
+            self.base.attributes.set('md5', self.generate_md5())
 
         return super().store(*args, **kwargs)
 
@@ -521,12 +507,12 @@ class CifData(SinglefileData):
                 self.source.get('source_md5', None) is not None and \
                 self.source['source_md5'] != md5sum:
             self.source = {}
-        self.set_attribute('md5', md5sum)
+        self.base.attributes.set('md5', md5sum)
 
         self._values = None
         self._ase = None
-        self.set_attribute('formulae', None)
-        self.set_attribute('spacegroup_numbers', None)
+        self.base.attributes.set('formulae', None)
+        self.base.attributes.set('spacegroup_numbers', None)
 
     def set_scan_type(self, scan_type):
         """
@@ -539,7 +525,7 @@ class CifData(SinglefileData):
         :param scan_type: Either 'standard' or 'flex' (see _scan_types)
         """
         if scan_type in CifData._SCAN_TYPES:
-            self.set_attribute('scan_type', scan_type)
+            self.base.attributes.set('scan_type', scan_type)
         else:
             raise ValueError(f'Got unknown scan_type {scan_type}')
 
@@ -551,7 +537,7 @@ class CifData(SinglefileData):
             or 'lazy' (defer parsing until needed)
         """
         if parse_policy in CifData._PARSE_POLICIES:
-            self.set_attribute('parse_policy', parse_policy)
+            self.base.attributes.set('parse_policy', parse_policy)
         else:
             raise ValueError(f'Got unknown parse_policy {parse_policy}')
 
@@ -707,7 +693,7 @@ class CifData(SinglefileData):
                 if tag in self.values[datablock].keys():
                     coords.extend(self.values[datablock][tag])
 
-        return not all([coord == '?' for coord in coords])
+        return not all(coord == '?' for coord in coords)
 
     @property
     def has_unknown_species(self):
@@ -732,7 +718,7 @@ class CifData(SinglefileData):
                 return None
 
             species = parse_formula(formula).keys()
-            if any([specie not in known_species for specie in species]):
+            if any(specie not in known_species for specie in species):
                 return True
 
         return False
@@ -785,10 +771,6 @@ class CifData(SinglefileData):
         If parsed values are present, a CIF string is created and written to file. If no parsed values are present, the
         CIF string is read from file.
         """
-        if self._values and not self.is_stored:
-            # Note: this overwrites the CIF file!
-            self.set_values(self._values)
-
         with self.open(mode='rb') as handle:
             return handle.read(), {}
 
@@ -817,7 +799,7 @@ class CifData(SinglefileData):
         super()._validate()
 
         try:
-            attr_md5 = self.get_attribute('md5')
+            attr_md5 = self.base.attributes.get('md5')
         except AttributeError:
             raise ValidationError("attribute 'md5' not set.")
         md5 = self.generate_md5()
