@@ -9,10 +9,10 @@
 ###########################################################################
 """Data plugin represeting an executable code to be wrapped and called through a `CalcJob` plugin."""
 import os
-import warnings
 
 from aiida.common import exceptions
-from aiida.common.warnings import AiidaDeprecationWarning
+from aiida.common.log import override_log_level
+
 from .data import Data
 
 __all__ = ('Code',)
@@ -61,21 +61,21 @@ class Code(Data):
         """
         Hide the code (prevents from showing it in the verdi code list)
         """
-        self.set_extra(self.HIDDEN_KEY, True)
+        self.base.extras.set(self.HIDDEN_KEY, True)
 
     def reveal(self):
         """
         Reveal the code (allows to show it in the verdi code list)
         By default, it is revealed
         """
-        self.set_extra(self.HIDDEN_KEY, False)
+        self.base.extras.set(self.HIDDEN_KEY, False)
 
     @property
     def hidden(self):
         """
         Determines whether the Code is hidden or not
         """
-        return self.get_extra(self.HIDDEN_KEY, False)
+        return self.base.extras.get(self.HIDDEN_KEY, False)
 
     def set_files(self, files):
         """
@@ -93,21 +93,13 @@ class Code(Data):
         for filename in files:
             if os.path.isfile(filename):
                 with open(filename, 'rb') as handle:
-                    self.put_object_from_filelike(handle, os.path.split(filename)[1], 'wb', encoding=None)
+                    self.base.repository.put_object_from_filelike(handle, os.path.split(filename)[1])
 
     def __str__(self):
         local_str = 'Local' if self.is_local() else 'Remote'
         computer_str = self.computer.label
         return f"{local_str} code '{self.label}' on {computer_str}, pk: {self.pk}, uuid: {self.uuid}"
 
-    def get_computer_name(self):
-        """Get label of this code's computer.
-
-        .. deprecated:: 1.4.0
-            Will be removed in `v2.0.0`, use the `self.get_computer_label()` method instead.
-        """
-        return self.get_computer_label()
-
     def get_computer_label(self):
         """Get label of this code's computer."""
         return 'repository' if self.is_local() else self.computer.label
@@ -136,19 +128,14 @@ class Code(Data):
         """
         if '@' in str(value):
             msg = "Code labels must not contain the '@' symbol"
-            raise exceptions.InputValidationError(msg)
+            raise ValueError(msg)
 
         super(Code, self.__class__).label.fset(self, value)  # pylint: disable=no-member
 
-    def relabel(self, new_label, raise_error=True):
+    def relabel(self, new_label):
         """Relabel this code.
 
         :param new_label: new code label
-        :param raise_error: Set to False in order to return a list of errors
-            instead of raising them.
-
-        .. deprecated:: 1.2.0
-            Will remove raise_error in `v2.0.0`. Use `try/except` instead.
         """
         # pylint: disable=unused-argument
         suffix = f'@{self.computer.label}'
@@ -165,7 +152,7 @@ class Code(Data):
         return f'{self.description}'
 
     @classmethod
-    def get_code_helper(cls, label, machinename=None):
+    def get_code_helper(cls, label, machinename=None, backend=None):
         """
         :param label: the code label identifying the code to load
         :param machinename: the machine name where code is setup
@@ -174,17 +161,17 @@ class Code(Data):
         :raise aiida.common.MultipleObjectsError: if the string cannot identify uniquely
             a code
         """
-        from aiida.common.exceptions import NotExistent, MultipleObjectsError
-        from aiida.orm.querybuilder import QueryBuilder
+        from aiida.common.exceptions import MultipleObjectsError, NotExistent
         from aiida.orm.computers import Computer
+        from aiida.orm.querybuilder import QueryBuilder
 
-        query = QueryBuilder()
+        query = QueryBuilder(backend=backend)
         query.append(cls, filters={'label': label}, project='*', tag='code')
         if machinename:
-            query.append(Computer, filters={'name': machinename}, with_node='code')
+            query.append(Computer, filters={'label': machinename}, with_node='code')
 
         if query.count() == 0:
-            raise NotExistent(f"'{label}' is not a valid code name.")
+            raise NotExistent(f"'{label}' is not a valid code label.")
         elif query.count() > 1:
             codes = query.all(flat=True)
             retstr = f"There are multiple codes with label '{label}', having IDs: "
@@ -206,7 +193,7 @@ class Code(Data):
 
         :raise aiida.common.NotExistent: if no code identified by the given string is found
         :raise aiida.common.MultipleObjectsError: if the string cannot identify uniquely a code
-        :raise aiida.common.InputValidationError: if neither a pk nor a label was passed in
+        :raise ValueError: if neither a pk nor a label was passed in
         """
         # pylint: disable=arguments-differ
         from aiida.orm.utils import load_code
@@ -226,7 +213,7 @@ class Code(Data):
             return cls.get_code_helper(label, machinename)
 
         else:
-            raise exceptions.InputValidationError('Pass either pk or code label (and machinename)')
+            raise ValueError('Pass either pk or code label (and machinename)')
 
     @classmethod
     def get_from_string(cls, code_string):
@@ -245,15 +232,15 @@ class Code(Data):
         :raise aiida.common.NotExistent: if no code identified by the given string is found
         :raise aiida.common.MultipleObjectsError: if the string cannot identify uniquely
             a code
-        :raise aiida.common.InputValidationError: if code_string is not of string type
+        :raise TypeError: if code_string is not of string type
 
         """
-        from aiida.common.exceptions import NotExistent, MultipleObjectsError, InputValidationError
+        from aiida.common.exceptions import MultipleObjectsError, NotExistent
 
         try:
             label, _, machinename = code_string.partition('@')
         except AttributeError:
-            raise InputValidationError('the provided code_string is not of valid string type')
+            raise TypeError('the provided code_string is not of valid string type')
 
         try:
             return cls.get_code_helper(label, machinename)
@@ -263,7 +250,7 @@ class Code(Data):
             raise MultipleObjectsError(f'{code_string} could not be uniquely resolved')
 
     @classmethod
-    def list_for_plugin(cls, plugin, labels=True):
+    def list_for_plugin(cls, plugin, labels=True, backend=None):
         """
         Return a list of valid code strings for a given plugin.
 
@@ -274,7 +261,7 @@ class Code(Data):
           otherwise a list of integers with the code PKs.
         """
         from aiida.orm.querybuilder import QueryBuilder
-        query = QueryBuilder()
+        query = QueryBuilder(backend=backend)
         query.append(cls, filters={'attributes.input_plugin': {'==': plugin}})
         valid_codes = query.all(flat=True)
 
@@ -295,32 +282,57 @@ class Code(Data):
                     'You have to set which file is the local executable '
                     'using the set_exec_filename() method'
                 )
-            if self.get_local_executable() not in self.list_object_names():
+            if self.get_local_executable() not in self.base.repository.list_object_names():
                 raise exceptions.ValidationError(
-                    "The local executable '{}' is not in the list of "
-                    'files of this code'.format(self.get_local_executable())
+                    f"The local executable '{self.get_local_executable()}' is not in the list of files of this code"
                 )
         else:
-            if self.list_object_names():
+            if self.base.repository.list_object_names():
                 raise exceptions.ValidationError('The code is remote but it has files inside')
             if not self.get_remote_computer():
                 raise exceptions.ValidationError('You did not specify a remote computer')
             if not self.get_remote_exec_path():
                 raise exceptions.ValidationError('You did not specify a remote executable')
 
+    def validate_remote_exec_path(self):
+        """Validate the ``remote_exec_path`` attribute.
+
+        Checks whether the executable exists on the remote computer if a transport can be opened to it. This method
+        is intentionally not called in ``_validate`` as to allow the creation of ``Code`` instances whose computers can
+        not yet be connected to and as to not require the overhead of opening transports in storing a new code.
+
+        :raises `~aiida.common.exceptions.ValidationError`: if no transport could be opened or if the defined executable
+            does not exist on the remote computer.
+        """
+        filepath = self.get_remote_exec_path()
+
+        try:
+            with override_log_level():  # Temporarily suppress noisy logging
+                with self.computer.get_transport() as transport:
+                    file_exists = transport.isfile(filepath)
+        except Exception:  # pylint: disable=broad-except
+            raise exceptions.ValidationError(
+                'Could not connect to the configured computer to determine whether the specified executable exists.'
+            )
+
+        if not file_exists:
+            raise exceptions.ValidationError(
+                f'the provided remote absolute path `{filepath}` does not exist on the computer.'
+            )
+
     def set_prepend_text(self, code):
         """
         Pass a string of code that will be put in the scheduler script before the
         execution of the code.
         """
-        self.set_attribute('prepend_text', str(code))
+        self.base.attributes.set('prepend_text', str(code))
 
     def get_prepend_text(self):
         """
         Return the code that will be put in the scheduler script before the
         execution, or an empty string if no pre-exec code was defined.
         """
-        return self.get_attribute('prepend_text', '')
+        return self.base.attributes.get('prepend_text', '')
 
     def set_input_plugin_name(self, input_plugin):
         """
@@ -328,29 +340,45 @@ class Code(Data):
         generation of a new calculation.
         """
         if input_plugin is None:
-            self.set_attribute('input_plugin', None)
+            self.base.attributes.set('input_plugin', None)
         else:
-            self.set_attribute('input_plugin', str(input_plugin))
+            self.base.attributes.set('input_plugin', str(input_plugin))
 
     def get_input_plugin_name(self):
         """
         Return the name of the default input plugin (or None if no input plugin
         was set.
         """
-        return self.get_attribute('input_plugin', None)
+        return self.base.attributes.get('input_plugin', None)
+
+    def set_use_double_quotes(self, use_double_quotes: bool):
+        """Set whether the command line invocation of this code should be escaped with double quotes.
+
+        :param use_double_quotes: True if to escape with double quotes, False otherwise.
+        """
+        from aiida.common.lang import type_check
+        type_check(use_double_quotes, bool)
+        self.base.attributes.set('use_double_quotes', use_double_quotes)
+
+    def get_use_double_quotes(self) -> bool:
+        """Return whether the command line invocation of this code should be escaped with double quotes.
+
+        :returns: True if to escape with double quotes, False otherwise which is also the default.
+        """
+        return self.base.attributes.get('use_double_quotes', False)
 
     def set_append_text(self, code):
         """
         Pass a string of code that will be put in the scheduler script after the
         execution of the code.
         """
-        self.set_attribute('append_text', str(code))
+        self.base.attributes.set('append_text', str(code))
 
     def get_append_text(self):
         """
         Return the postexec_code, or an empty string if no post-exec code was defined.
         """
-        return self.get_attribute('append_text', '')
+        return self.base.attributes.get('append_text', '')
 
     def set_local_executable(self, exec_name):
         """
@@ -358,10 +386,10 @@ class Code(Data):
         Implicitly set the code as local.
         """
         self._set_local()
-        self.set_attribute('local_executable', exec_name)
+        self.base.attributes.set('local_executable', exec_name)
 
     def get_local_executable(self):
-        return self.get_attribute('local_executable', '')
+        return self.base.attributes.get('local_executable', '')
 
     def set_remote_computer_exec(self, remote_computer_exec):
         """
@@ -376,9 +404,7 @@ class Code(Data):
 
         if (not isinstance(remote_computer_exec, (list, tuple)) or len(remote_computer_exec) != 2):
             raise ValueError(
-                'remote_computer_exec must be a list or tuple '
-                'of length 2, with machine and executable '
-                'name'
+                'remote_computer_exec must be a list or tuple of length 2, with machine and executable name'
             )
 
         computer, remote_exec_path = tuple(remote_computer_exec)
@@ -390,12 +416,12 @@ class Code(Data):
 
         self._set_remote()
         self.computer = computer
-        self.set_attribute('remote_exec_path', remote_exec_path)
+        self.base.attributes.set('remote_exec_path', remote_exec_path)
 
     def get_remote_exec_path(self):
         if self.is_local():
             raise ValueError('The code is local')
-        return self.get_attribute('remote_exec_path', '')
+        return self.base.attributes.get('remote_exec_path', '')
 
     def get_remote_computer(self):
         if self.is_local():
@@ -411,10 +437,10 @@ class Code(Data):
 
         It also deletes the flags related to the local case (if any)
         """
-        self.set_attribute('is_local', True)
+        self.base.attributes.set('is_local', True)
         self.computer = None
         try:
-            self.delete_attribute('remote_exec_path')
+            self.base.attributes.delete('remote_exec_path')
         except AttributeError:
             pass
 
@@ -426,9 +452,9 @@ class Code(Data):
 
         It also deletes the flags related to the local case (if any)
         """
-        self.set_attribute('is_local', False)
+        self.base.attributes.set('is_local', False)
         try:
-            self.delete_attribute('local_executable')
+            self.base.attributes.delete('local_executable')
         except AttributeError:
             pass
 
@@ -437,7 +463,7 @@ class Code(Data):
         Return True if the code is 'local', False if it is 'remote' (see also documentation
         of the set_local and set_remote functions).
         """
-        return self.get_attribute('is_local', None)
+        return self.base.attributes.get('is_local', None)
 
     def can_run_on(self, computer):
         """
@@ -455,7 +481,7 @@ class Code(Data):
             return True
 
         type_check(computer, orm.Computer)
-        return computer.id == self.get_remote_computer().id
+        return computer.pk == self.get_remote_computer().pk
 
     def get_execname(self):
         """
@@ -495,54 +521,3 @@ class Code(Data):
         builder.code = self
 
         return builder
-
-    def get_full_text_info(self, verbose=False):
-        """Return a list of lists with a human-readable detailed information on this code.
-
-        .. deprecated:: 1.4.0
-            Will be removed in `v2.0.0`.
-
-        :return: list of lists where each entry consists of two elements: a key and a value
-        """
-        warnings.warn('this property is deprecated', AiidaDeprecationWarning)  # pylint: disable=no-member
-        from aiida.repository import FileType
-
-        result = []
-        result.append(['PK', self.pk])
-        result.append(['UUID', self.uuid])
-        result.append(['Label', self.label])
-        result.append(['Description', self.description])
-        result.append(['Default plugin', self.get_input_plugin_name()])
-
-        if verbose:
-            result.append(['Calculations', len(self.get_outgoing().all())])
-
-        if self.is_local():
-            result.append(['Type', 'local'])
-            result.append(['Exec name', self.get_execname()])
-            result.append(['List of files/folders:', ''])
-            for obj in self.list_objects():
-                if obj.file_type == FileType.DIRECTORY:
-                    result.append(['directory', obj.name])
-                else:
-                    result.append(['file', obj.name])
-        else:
-            result.append(['Type', 'remote'])
-            result.append(['Remote machine', self.get_remote_computer().label])
-            result.append(['Remote absolute path', self.get_remote_exec_path()])
-
-        if self.get_prepend_text().strip():
-            result.append(['Prepend text', ''])
-            for line in self.get_prepend_text().split('\n'):
-                result.append(['', line])
-        else:
-            result.append(['Prepend text', 'No prepend text'])
-
-        if self.get_append_text().strip():
-            result.append(['Append text', ''])
-            for line in self.get_append_text().split('\n'):
-                result.append(['', line])
-        else:
-            result.append(['Append text', 'No append text'])
-
-        return result
