@@ -8,12 +8,12 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """`Data` sub class to represent a dictionary."""
-
 import copy
 
 from aiida.common import exceptions
-from .data import Data
+
 from .base import to_aiida_type
+from .data import Data
 
 __all__ = ('Dict',)
 
@@ -46,32 +46,41 @@ class Dict(Data):
     Finally, all dictionary mutations will be forbidden once the node is stored.
     """
 
-    def __init__(self, **kwargs):
-        """Store a dictionary as a `Node` instance.
+    def __init__(self, value=None, **kwargs):
+        """Initialise a ``Dict`` node instance.
 
-        Usual rules for attribute names apply, in particular, keys cannot start with an underscore, or a `ValueError`
+        Usual rules for attribute names apply, in particular, keys cannot start with an underscore, or a ``ValueError``
         will be raised.
 
         Initial attributes can be changed, deleted or added as long as the node is not stored.
 
-        :param dict: the dictionary to set
+        :param value: dictionary to initialise the ``Dict`` node from
         """
-        dictionary = kwargs.pop('dict', None)
+        dictionary = value or kwargs.pop('dict', None)
         super().__init__(**kwargs)
         if dictionary:
             self.set_dict(dictionary)
 
     def __getitem__(self, key):
         try:
-            return self.get_attribute(key)
+            return self.base.attributes.get(key)
         except AttributeError as exc:
             raise KeyError from exc
 
     def __setitem__(self, key, value):
-        self.set_attribute(key, value)
+        self.base.attributes.set(key, value)
+
+    def __eq__(self, other):
+        if isinstance(other, Dict):
+            return self.get_dict() == other.get_dict()
+        return self.get_dict() == other
+
+    def __contains__(self, key: str) -> bool:
+        """Return whether the node contains a key."""
+        return key in self.base.attributes
 
     def set_dict(self, dictionary):
-        """ Replace the current dictionary with another one.
+        """Replace the current dictionary with another one.
 
         :param dictionary: dictionary to set
         """
@@ -79,14 +88,14 @@ class Dict(Data):
 
         try:
             # Clear existing attributes and set the new dictionary
-            self.clear_attributes()
+            self.base.attributes.clear()
             self.update_dict(dictionary)
         except exceptions.ModificationNotAllowed:  # pylint: disable=try-except-raise
             # I reraise here to avoid to go in the generic 'except' below that would raise the same exception again
             raise
         except Exception:
             # Try to restore the old data
-            self.clear_attributes()
+            self.base.attributes.clear()
             self.update_dict(dictionary_backup)
             raise
 
@@ -98,23 +107,28 @@ class Dict(Data):
         :param dictionary: a dictionary with the keys to substitute
         """
         for key, value in dictionary.items():
-            self.set_attribute(key, value)
+            self.base.attributes.set(key, value)
 
     def get_dict(self):
         """Return a dictionary with the parameters currently set.
 
         :return: dictionary
         """
-        return dict(self.attributes)
+        return dict(self.base.attributes.all)
 
     def keys(self):
         """Iterator of valid keys stored in the Dict object.
 
         :return: iterator over the keys of the current dictionary
         """
-        for key in self.attributes.keys():
+        for key in self.base.attributes.keys():
             yield key
 
+    def items(self):
+        """Iterator of all items stored in the Dict node."""
+        for key, value in self.base.attributes.items():
+            yield key, value
+
     @property
     def dict(self):
         """Return an instance of `AttributeManager` that transforms the dictionary into an attribute dict.
@@ -129,4 +143,4 @@ class Dict(Data):
 
 @to_aiida_type.register(dict)
 def _(value):
-    return Dict(dict=value)
+    return Dict(value)
