@@ -9,17 +9,14 @@
 ###########################################################################
 """Module with `Node` sub class for calculation job processes."""
 import datetime
-from typing import Any, AnyStr, Dict, List, Optional, Sequence, Tuple, Type, Union
-from typing import TYPE_CHECKING
-import warnings
+from typing import TYPE_CHECKING, Any, AnyStr, Dict, List, Optional, Sequence, Tuple, Type, Union
 
 from aiida.common import exceptions
 from aiida.common.datastructures import CalcJobState
 from aiida.common.lang import classproperty
 from aiida.common.links import LinkType
-from aiida.common.folders import Folder
-from aiida.common.warnings import AiidaDeprecationWarning
 
+from ..process import ProcessNodeCaching
 from .calculation import CalculationNode
 
 if TYPE_CHECKING:
@@ -35,25 +32,52 @@ if TYPE_CHECKING:
 __all__ = ('CalcJobNode',)
 
 
+class CalcJobNodeCaching(ProcessNodeCaching):
+    """Interface to control caching of a node instance."""
+
+    def _get_objects_to_hash(self) -> List[Any]:
+        """Return a list of objects which should be included in the hash.
+
+        This method is purposefully overridden from the base `Node` class, because we do not want to include the
+        repository folder in the hash. The reason is that the hash of this node is computed in the `store` method, at
+        which point the input files that will be stored in the repository have not yet been generated. Including these
+        anyway in the computation of the hash would mean that the hash of the node would change as soon as the process
+        has started and the input files have been written to the repository.
+        """
+        from importlib import import_module
+        objects = [
+            import_module(self._node.__module__.split('.', 1)[0]).__version__,
+            {
+                key: val
+                for key, val in self._node.base.attributes.items()
+                if key not in self._node._hash_ignored_attributes and key not in self._node._updatable_attributes  # pylint: disable=unsupported-membership-test,protected-access
+            },
+            self._node.computer.uuid if self._node.computer is not None else None,  # pylint: disable=no-member
+            {
+                entry.link_label: entry.node.base.caching.get_hash()
+                for entry in self._node.base.links.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
+                if entry.link_label not in self._hash_ignored_inputs
+            }
+        ]
+        return objects
+
+
 class CalcJobNode(CalculationNode):
     """ORM class for all nodes representing the execution of a CalcJob."""
 
     # pylint: disable=too-many-public-methods
 
     CALC_JOB_STATE_KEY = 'state'
+    IMMIGRATED_KEY = 'imported'
     REMOTE_WORKDIR_KEY = 'remote_workdir'
     RETRIEVE_LIST_KEY = 'retrieve_list'
     RETRIEVE_TEMPORARY_LIST_KEY = 'retrieve_temporary_list'
-    RETRIEVE_SINGLE_FILE_LIST_KEY = 'retrieve_singlefile_list'
     SCHEDULER_JOB_ID_KEY = 'job_id'
     SCHEDULER_STATE_KEY = 'scheduler_state'
     SCHEDULER_LAST_CHECK_TIME_KEY = 'scheduler_lastchecktime'
     SCHEDULER_LAST_JOB_INFO_KEY = 'last_job_info'
     SCHEDULER_DETAILED_JOB_INFO_KEY = 'detailed_job_info'
 
-    # Base path within the repository where to put objects by default
-    _repository_base_path = 'raw_input'
-
     # An optional entry point for a CalculationTools instance
     _tools = None
 
@@ -68,22 +92,26 @@ class CalcJobNode(CalculationNode):
 
         :return: CalculationTools instance
         """
-        from aiida.plugins.entry_point import is_valid_entry_point_string, get_entry_point_from_string, load_entry_point
+        from aiida.plugins.entry_point import get_entry_point_from_string, is_valid_entry_point_string, load_entry_point
         from aiida.tools.calculations import CalculationTools
 
         if self._tools is None:
             entry_point_string = self.process_type
 
-            if is_valid_entry_point_string(entry_point_string):
+            if entry_point_string and is_valid_entry_point_string(entry_point_string):
                 entry_point = get_entry_point_from_string(entry_point_string)
 
                 try:
-                    tools_class = load_entry_point('aiida.tools.calculations', entry_point.name)
+                    tools_class = load_entry_point(
+                        'aiida.tools.calculations',
+                        entry_point.name  # type: ignore[attr-defined]
+                    )
                     self._tools = tools_class(self)
                 except exceptions.EntryPointError as exception:
                     self._tools = CalculationTools(self)
+                    entry_point_name = entry_point.name  # type: ignore[attr-defined]
                     self.logger.warning(
-                        f'could not load the calculation tools entry point {entry_point.name}: {exception}'
+                        f'could not load the calculation tools entry point {entry_point_name}: {exception}'
                     )
 
         return self._tools
@@ -92,10 +120,10 @@ class CalcJobNode(CalculationNode):
     def _updatable_attributes(cls) -> Tuple[str, ...]:  # pylint: disable=no-self-argument
         return super()._updatable_attributes + (
             cls.CALC_JOB_STATE_KEY,
+            cls.IMMIGRATED_KEY,
             cls.REMOTE_WORKDIR_KEY,
             cls.RETRIEVE_LIST_KEY,
             cls.RETRIEVE_TEMPORARY_LIST_KEY,
-            cls.RETRIEVE_SINGLE_FILE_LIST_KEY,
             cls.SCHEDULER_JOB_ID_KEY,
             cls.SCHEDULER_STATE_KEY,
             cls.SCHEDULER_LAST_CHECK_TIME_KEY,
@@ -114,32 +142,6 @@ class CalcJobNode(CalculationNode):
             'max_memory_kb',
         )
 
-    def _get_objects_to_hash(self) -> List[Any]:
-        """Return a list of objects which should be included in the hash.
-
-        This method is purposefully overridden from the base `Node` class, because we do not want to include the
-        repository folder in the hash. The reason is that the hash of this node is computed in the `store` method, at
-        which point the input files that will be stored in the repository have not yet been generated. Including these
-        anyway in the computation of the hash would mean that the hash of the node would change as soon as the process
-        has started and the input files have been written to the repository.
-        """
-        from importlib import import_module
-        objects = [
-            import_module(self.__module__.split('.', 1)[0]).__version__,  # type: ignore[attr-defined]
-            {
-                key: val
-                for key, val in self.attributes_items()
-                if key not in self._hash_ignored_attributes and key not in self._updatable_attributes  # pylint: disable=unsupported-membership-test
-            },
-            self.computer.uuid if self.computer is not None else None,  # pylint: disable=no-member
-            {
-                entry.link_label: entry.node.get_hash()
-                for entry in self.get_incoming(link_type=(LinkType.INPUT_CALC, LinkType.INPUT_WORK))
-                if entry.link_label not in self._hash_ignored_inputs
-            }
-        ]
-        return objects
-
     def get_builder_restart(self) -> 'ProcessBuilder':
         """Return a `ProcessBuilder` that is ready to relaunch the same `CalcJob` that created this node.
 
@@ -153,26 +155,12 @@ class CalcJobNode(CalculationNode):
         """
         builder = super().get_builder_restart()
         builder.metadata.options = self.get_options()  # type: ignore[attr-defined]
-
         return builder
 
     @property
-    def _raw_input_folder(self) -> Folder:
-        """
-        Get the input folder object.
-
-        :return: the input folder object.
-        :raise: NotExistent: if the raw folder hasn't been created yet
-        """
-        from aiida.common.exceptions import NotExistent
-
-        assert self._repository is not None, 'repository not initialised'
-
-        return_folder = self._repository._get_base_folder()  # pylint: disable=protected-access
-        if return_folder.exists():
-            return return_folder
-
-        raise NotExistent('the `_raw_input_folder` has not yet been created')
+    def is_imported(self) -> bool:
+        """Return whether the calculation job was imported instead of being an actual run."""
+        return self.base.attributes.get(self.IMMIGRATED_KEY, None) is True
 
     def get_option(self, name: str) -> Optional[Any]:
         """
@@ -182,7 +170,7 @@ class CalcJobNode(CalculationNode):
         :return: the option value or None
         :raises: ValueError for unknown option
         """
-        return self.get_attribute(name, None)
+        return self.base.attributes.get(name, None)
 
     def set_option(self, name: str, value: Any) -> None:
         """
@@ -193,7 +181,7 @@ class CalcJobNode(CalculationNode):
         :raises: ValueError for unknown option
         :raises: TypeError for values with invalid type
         """
-        self.set_attribute(name, value)
+        self.base.attributes.set(name, value)
 
     def get_options(self) -> Dict[str, Any]:
         """
@@ -228,7 +216,7 @@ class CalcJobNode(CalculationNode):
 
         :return: instance of `aiida.common.datastructures.CalcJobState` or `None` if invalid value, or not set
         """
-        state = self.get_attribute(self.CALC_JOB_STATE_KEY, None)
+        state = self.base.attributes.get(self.CALC_JOB_STATE_KEY, None)
 
         try:
             state = CalcJobState(state)
@@ -245,12 +233,12 @@ class CalcJobNode(CalculationNode):
         if not isinstance(state, CalcJobState):
             raise ValueError(f'{state} is not a valid CalcJobState')
 
-        self.set_attribute(self.CALC_JOB_STATE_KEY, state.value)
+        self.base.attributes.set(self.CALC_JOB_STATE_KEY, state.value)
 
     def delete_state(self) -> None:
         """Delete the calculation job state attribute if it exists."""
         try:
-            self.delete_attribute(self.CALC_JOB_STATE_KEY)
+            self.base.attributes.delete(self.CALC_JOB_STATE_KEY)
         except AttributeError:
             pass
 
@@ -259,14 +247,14 @@ class CalcJobNode(CalculationNode):
 
         :param remote_workdir: absolute filepath to the remote working directory
         """
-        self.set_attribute(self.REMOTE_WORKDIR_KEY, remote_workdir)
+        self.base.attributes.set(self.REMOTE_WORKDIR_KEY, remote_workdir)
 
     def get_remote_workdir(self) -> Optional[str]:
         """Return the path to the remote (on cluster) scratch folder of the calculation.
 
         :return: a string with the remote path
         """
-        return self.get_attribute(self.REMOTE_WORKDIR_KEY, None)
+        return self.base.attributes.get(self.REMOTE_WORKDIR_KEY, None)
 
     @staticmethod
     def _validate_retrieval_directive(directives: Sequence[Union[str, Tuple[str, str, str]]]) -> None:
@@ -306,14 +294,14 @@ class CalcJobNode(CalculationNode):
         :param retrieve_list: list or tuple of with filepath directives
         """
         self._validate_retrieval_directive(retrieve_list)
-        self.set_attribute(self.RETRIEVE_LIST_KEY, retrieve_list)
+        self.base.attributes.set(self.RETRIEVE_LIST_KEY, retrieve_list)
 
     def get_retrieve_list(self) -> Optional[Sequence[Union[str, Tuple[str, str, str]]]]:
         """Return the list of files/directories to be retrieved on the cluster after the calculation has completed.
 
         :return: a list of file directives
         """
-        return self.get_attribute(self.RETRIEVE_LIST_KEY, None)
+        return self.base.attributes.get(self.RETRIEVE_LIST_KEY, None)
 
     def set_retrieve_temporary_list(self, retrieve_temporary_list: Sequence[Union[str, Tuple[str, str, str]]]) -> None:
         """Set the retrieve temporary list.
@@ -324,54 +312,14 @@ class CalcJobNode(CalculationNode):
         :param retrieve_temporary_list: list or tuple of with filepath directives
         """
         self._validate_retrieval_directive(retrieve_temporary_list)
-        self.set_attribute(self.RETRIEVE_TEMPORARY_LIST_KEY, retrieve_temporary_list)
+        self.base.attributes.set(self.RETRIEVE_TEMPORARY_LIST_KEY, retrieve_temporary_list)
 
     def get_retrieve_temporary_list(self) -> Optional[Sequence[Union[str, Tuple[str, str, str]]]]:
         """Return list of files to be retrieved from the cluster which will be available during parsing.
 
         :return: a list of file directives
         """
-        return self.get_attribute(self.RETRIEVE_TEMPORARY_LIST_KEY, None)
-
-    def set_retrieve_singlefile_list(self, retrieve_singlefile_list):
-        """Set the retrieve singlefile list.
-
-        The files will be stored as `SinglefileData` instances and added as output nodes to this calculation node.
-        The format of a single file directive is a tuple or list of length 3 with the following entries:
-
-            1. the link label under which the file should be added
-            2. the `SinglefileData` class or sub class to use to store
-            3. the filepath relative to the remote working directory of the calculation
-
-        :param retrieve_singlefile_list: list or tuple of single file directives
-
-        .. deprecated:: 1.0.0
-
-            Will be removed in `v2.0.0`.
-            Use :meth:`~aiida.orm.nodes.process.calculation.calcjob.CalcJobNode.set_retrieve_temporary_list` instead.
-
-        """
-        warnings.warn('method is deprecated, use `set_retrieve_temporary_list` instead', AiidaDeprecationWarning)  # pylint: disable=no-member
-
-        if not isinstance(retrieve_singlefile_list, (tuple, list)):
-            raise TypeError('retrieve_singlefile_list has to be a list or tuple')
-
-        for j in retrieve_singlefile_list:
-            if not isinstance(j, (tuple, list)) or not all(isinstance(i, str) for i in j):
-                raise ValueError('You have to pass a list (or tuple) of lists of strings as retrieve_singlefile_list')
-
-        self.set_attribute(self.RETRIEVE_SINGLE_FILE_LIST_KEY, retrieve_singlefile_list)
-
-    def get_retrieve_singlefile_list(self):
-        """Return the list of files to be retrieved on the cluster after the calculation has completed.
-
-        :return: list of single file retrieval directives
-
-        .. deprecated:: 1.0.0
-            Will be removed in `v2.0.0`, use
-            :meth:`aiida.orm.nodes.process.calculation.calcjob.CalcJobNode.get_retrieve_temporary_list` instead.
-        """
-        return self.get_attribute(self.RETRIEVE_SINGLE_FILE_LIST_KEY, None)
+        return self.base.attributes.get(self.RETRIEVE_TEMPORARY_LIST_KEY, None)
 
     def set_job_id(self, job_id: Union[int, str]) -> None:
         """Set the job id that was assigned to the calculation by the scheduler.
@@ -380,14 +328,14 @@ class CalcJobNode(CalculationNode):
 
         :param job_id: the id assigned by the scheduler after submission
         """
-        return self.set_attribute(self.SCHEDULER_JOB_ID_KEY, str(job_id))
+        return self.base.attributes.set(self.SCHEDULER_JOB_ID_KEY, str(job_id))
 
     def get_job_id(self) -> Optional[str]:
         """Return job id that was assigned to the calculation by the scheduler.
 
         :return: the string representation of the scheduler job id
         """
-        return self.get_attribute(self.SCHEDULER_JOB_ID_KEY, None)
+        return self.base.attributes.get(self.SCHEDULER_JOB_ID_KEY, None)
 
     def set_scheduler_state(self, state: 'JobState') -> None:
         """Set the scheduler state.
@@ -400,8 +348,8 @@ class CalcJobNode(CalculationNode):
         if not isinstance(state, JobState):
             raise ValueError(f'scheduler state should be an instance of JobState, got: {state}')
 
-        self.set_attribute(self.SCHEDULER_STATE_KEY, state.value)
-        self.set_attribute(self.SCHEDULER_LAST_CHECK_TIME_KEY, timezone.datetime_to_isoformat(timezone.now()))
+        self.base.attributes.set(self.SCHEDULER_STATE_KEY, state.value)
+        self.base.attributes.set(self.SCHEDULER_LAST_CHECK_TIME_KEY, timezone.now().isoformat())
 
     def get_scheduler_state(self) -> Optional['JobState']:
         """Return the status of the calculation according to the cluster scheduler.
@@ -410,7 +358,7 @@ class CalcJobNode(CalculationNode):
         """
         from aiida.schedulers.datastructures import JobState
 
-        state = self.get_attribute(self.SCHEDULER_STATE_KEY, None)
+        state = self.base.attributes.get(self.SCHEDULER_STATE_KEY, None)
 
         if state is None:
             return state
@@ -422,11 +370,10 @@ class CalcJobNode(CalculationNode):
 
         :return: a datetime object or None
         """
-        from aiida.common import timezone
-        value = self.get_attribute(self.SCHEDULER_LAST_CHECK_TIME_KEY, None)
+        value = self.base.attributes.get(self.SCHEDULER_LAST_CHECK_TIME_KEY, None)
 
         if value is not None:
-            value = timezone.isoformat_to_datetime(value)
+            value = datetime.datetime.fromisoformat(value)
 
         return value
 
@@ -435,7 +382,7 @@ class CalcJobNode(CalculationNode):
 
         :param detailed_job_info: a dictionary with metadata with the accounting of a completed job
         """
-        self.set_attribute(self.SCHEDULER_DETAILED_JOB_INFO_KEY, detailed_job_info)
+        self.base.attributes.set(self.SCHEDULER_DETAILED_JOB_INFO_KEY, detailed_job_info)
 
     def get_detailed_job_info(self) -> Optional[dict]:
         """Return the detailed job info dictionary.
@@ -444,14 +391,14 @@ class CalcJobNode(CalculationNode):
 
         :return: the dictionary with detailed job info if defined or None
         """
-        return self.get_attribute(self.SCHEDULER_DETAILED_JOB_INFO_KEY, None)
+        return self.base.attributes.get(self.SCHEDULER_DETAILED_JOB_INFO_KEY, None)
 
     def set_last_job_info(self, last_job_info: 'JobInfo') -> None:
         """Set the last job info.
 
         :param last_job_info: a `JobInfo` object
         """
-        self.set_attribute(self.SCHEDULER_LAST_JOB_INFO_KEY, last_job_info.get_dict())
+        self.base.attributes.set(self.SCHEDULER_LAST_JOB_INFO_KEY, last_job_info.get_dict())
 
     def get_last_job_info(self) -> Optional['JobInfo']:
         """Return the last information asked to the scheduler about the status of the job.
@@ -466,7 +413,7 @@ class CalcJobNode(CalculationNode):
         """
         from aiida.schedulers.datastructures import JobInfo
 
-        last_job_info_dictserialized = self.get_attribute(self.SCHEDULER_LAST_JOB_INFO_KEY, None)
+        last_job_info_dictserialized = self.base.attributes.get(self.SCHEDULER_LAST_JOB_INFO_KEY, None)
 
         if last_job_info_dictserialized is not None:
             job_info = JobInfo.load_from_dict(last_job_info_dictserialized)
@@ -485,7 +432,7 @@ class CalcJobNode(CalculationNode):
         if computer is None:
             raise exceptions.NotExistent('No computer has been set for this calculation')
 
-        return computer.get_authinfo(self.user)
+        return computer.get_authinfo(self.user)  # pylint: disable=no-member
 
     def get_transport(self) -> 'Transport':
         """Return the transport for this calculation.
@@ -521,7 +468,8 @@ class CalcJobNode(CalculationNode):
         """
         from aiida.orm import FolderData
         try:
-            return self.get_outgoing(node_class=FolderData, link_label_filter=self.link_label_retrieved).one().node
+            return self.base.links.get_outgoing(node_class=FolderData,
+                                                link_label_filter=self.link_label_retrieved).one().node
         except ValueError:
             return None
 
@@ -551,7 +499,7 @@ class CalcJobNode(CalculationNode):
             return None
 
         try:
-            stdout = retrieved_node.get_object_content(filename)
+            stdout = retrieved_node.base.repository.get_object_content(filename)
         except IOError:
             stdout = None
 
@@ -569,7 +517,7 @@ class CalcJobNode(CalculationNode):
             return None
 
         try:
-            stderr = retrieved_node.get_object_content(filename)
+            stderr = retrieved_node.base.repository.get_object_content(filename)
         except IOError:
             stderr = None
 
