@@ -8,12 +8,10 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Manage code objects with lazy loading of the db env"""
-
 import enum
 import os
 
 from aiida.cmdline.utils.decorators import with_dbenv
-from aiida.cmdline.params.types.plugin import PluginParamType
 from aiida.common.utils import ErrorAccumulator
 
 
@@ -64,7 +62,8 @@ class CodeBuilder:
 
         code.label = self._get_and_count('label', used)
         code.description = self._get_and_count('description', used)
-        code.set_input_plugin_name(self._get_and_count('input_plugin', used).name)
+        code.set_input_plugin_name(self._get_and_count('input_plugin', used))
+        code.set_use_double_quotes(self._get_and_count('use_double_quotes', used))
         code.set_prepend_text(self._get_and_count('prepend_text', used))
         code.set_append_text(self._get_and_count('append_text', used))
 
@@ -100,6 +99,7 @@ class CodeBuilder:
         spec['label'] = code.label
         spec['description'] = code.description
         spec['input_plugin'] = code.get_input_plugin_name()
+        spec['use_double_quotes'] = code.get_use_double_quotes()
         spec['prepend_text'] = code.get_prepend_text()
         spec['append_text'] = code.get_append_text()
 
@@ -155,9 +155,8 @@ class CodeBuilder:
 
         Checks compatibility with other code attributes.
         """
-        # store only string of input plugin
-        if key == 'input_plugin' and isinstance(value, PluginParamType):
-            value = value.name
+        if key == 'description' and value is None:
+            value = ''
 
         backup = self._code_spec.copy()
         self._code_spec[key] = value
@@ -195,7 +194,7 @@ class CodeBuilder:
         if messages:
             raise self.CodeValidationError(f'{messages}')
 
-    class CodeValidationError(Exception):
+    class CodeValidationError(ValueError):
         """
         A CodeBuilder instance may raise this
 
