@@ -14,11 +14,17 @@ the data structure to be filled for job submission (JobTemplate), and
 the data structure that is returned when querying for jobs in the scheduler
 (JobInfo).
 """
+from __future__ import annotations
+
 import abc
+from dataclasses import dataclass, field
+from datetime import datetime, timezone
 import enum
+import json
 
 from aiida.common import AIIDA_LOGGER
 from aiida.common.extendeddicts import AttributeDict, DefaultFieldsAttributeDict
+from aiida.common.timezone import make_aware, timezone_from_name
 
 SCHEDULER_LOGGER = AIIDA_LOGGER.getChild('scheduler')
 
@@ -62,7 +68,8 @@ class JobResource(DefaultFieldsAttributeDict, metaclass=abc.ABCMeta):
     """
     _default_fields = tuple()
 
-    @abc.abstractclassmethod
+    @classmethod
+    @abc.abstractmethod
     def validate_resources(cls, **kwargs):
         """Validate the resources against the job resource class of this scheduler.
 
@@ -76,7 +83,8 @@ class JobResource(DefaultFieldsAttributeDict, metaclass=abc.ABCMeta):
         """Return a list of valid keys to be passed to the constructor."""
         return list(cls._default_fields)
 
-    @abc.abstractclassmethod
+    @classmethod
+    @abc.abstractmethod
     def accepts_default_mpiprocs_per_machine(cls):
         """Return True if this subclass accepts a `default_mpiprocs_per_machine` key, False otherwise."""
 
@@ -246,6 +254,9 @@ class JobTemplate(DefaultFieldsAttributeDict):  # pylint: disable=too-many-insta
       * ``rerunnable``: if the job is rerunnable (boolean)
       * ``job_environment``: a dictionary with environment variables to set
         before the execution of the code.
+      * ``environment_variables_double_quotes``: if set to True, use double quotes
+        instead of single quotes to escape the environment variables specified
+        in ``environment_variables``.
       * ``working_directory``: the working directory for this job. During
         submission, the transport will first do a 'chdir' to this directory,
         and then possibly set a scheduler parameter, if this is supported
@@ -292,7 +303,7 @@ class JobTemplate(DefaultFieldsAttributeDict):  # pylint: disable=too-many-insta
       * ``append_text``: a (possibly multi-line) string to be inserted
         in the scheduler script after the main execution line
       * ``import_sys_environment``: import the system environment variables
-      * ``codes_info``: a list of aiida.common.datastructures.CalcInfo objects.
+      * ``codes_info``: a list of aiida.scheduler.datastructures.JobTemplateCodeInfo objects.
         Each contains the information necessary to run a single code. At the
         moment, it can contain:
 
@@ -326,6 +337,7 @@ class JobTemplate(DefaultFieldsAttributeDict):  # pylint: disable=too-many-insta
         'submit_as_hold',
         'rerunnable',
         'job_environment',
+        'environment_variables_double_quotes',
         'working_directory',
         'email',
         'email_on_started',
@@ -350,6 +362,32 @@ class JobTemplate(DefaultFieldsAttributeDict):  # pylint: disable=too-many-insta
     )
 
 
+@dataclass
+class JobTemplateCodeInfo:
+    """
+    Data structure to communicate to a `Scheduler` how a code should be run in submit script.
+
+    `Scheduler.get_submit_script` will pass a list of these objects to `Scheduler._get_run_line` which
+    should build up the code execution line based on the parameters specified in this dataclass.
+
+    :param preprend_cmdline_params: list of unescaped command line arguments that are to be prepended to the executable.
+    :param cmdline_params: list of unescaped command line parameters.
+    :param use_double_quotes: list of two booleans. If true, use double quotes to escape command line arguments. The
+        first value applies to `prepend_cmdline_params` and the second to `cmdline_params`.
+    :param stdin_name: filename of the the stdin file descriptor.
+    :param stdout_name: filename of the the `stdout` file descriptor.
+    :param stderr_name: filename of the the `stderr` file descriptor.
+    :param join_files: boolean, if true, `stderr` should be redirected to `stdout`.
+    """
+    prepend_cmdline_params: list[str] = field(default_factory=list)
+    cmdline_params: list[str] = field(default_factory=list)
+    use_double_quotes: list[bool] = field(default_factory=lambda: [False, False])
+    stdin_name: None | str = None
+    stdout_name: None | str = None
+    stderr_name: None | str = None
+    join_files: bool = False
+
+
 class MachineInfo(DefaultFieldsAttributeDict):
     """
     Similarly to what is defined in the DRMAA v.2 as SlotInfo; this identifies
@@ -456,23 +494,17 @@ class JobInfo(DefaultFieldsAttributeDict):  # pylint: disable=too-many-instance-
         :param value: The value to serialise
         :return: The serialised value
         """
-
-        import datetime
-        import pytz
-
         if value is None:
             return value
 
-        if not isinstance(value, datetime.datetime):
+        if not isinstance(value, datetime):
             raise TypeError('Invalid type for the date, should be a datetime')
 
-        # is_naive check from django.utils.timezone
         if value.tzinfo is None or value.tzinfo.utcoffset(value) is None:
             SCHEDULER_LOGGER.debug('Datetime to serialize in JobInfo is naive, this should be fixed!')
-            # v = v.replace(tzinfo = pytz.utc)
             return {'date': value.strftime('%Y-%m-%dT%H:%M:%S.%f'), 'timezone': None}
 
-        return {'date': value.astimezone(pytz.utc).strftime('%Y-%m-%dT%H:%M:%S.%f'), 'timezone': 'UTC'}
+        return {'date': value.astimezone(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S.%f'), 'timezone': 'UTC'}
 
     @staticmethod
     def _deserialize_date(value):
@@ -481,21 +513,19 @@ class JobInfo(DefaultFieldsAttributeDict):  # pylint: disable=too-many-instance-
         :param value: The date vlue
         :return: The deserialised date
         """
-        import datetime
-        import pytz
-
         if value is None:
             return value
 
         if value['timezone'] is None:
             # naive date
-            return datetime.datetime.strptime(value['date'], '%Y-%m-%dT%H:%M:%S.%f')
+            return datetime.strptime(value['date'], '%Y-%m-%dT%H:%M:%S.%f')
         if value['timezone'] == 'UTC':
-            return datetime.datetime.strptime(value['date'], '%Y-%m-%dT%H:%M:%S.%f').replace(tzinfo=pytz.utc)
+            return make_aware(datetime.strptime(value['date'], '%Y-%m-%dT%H:%M:%S.%f'), timezone.utc)
 
-        # Try your best
-        return datetime.datetime.strptime(value['date'],
-                                          '%Y-%m-%dT%H:%M:%S.%f').replace(tzinfo=pytz.timezone(value['timezone']))
+        # Try your best to guess the timezone from the name.
+        return make_aware(
+            datetime.strptime(value['date'], '%Y-%m-%dT%H:%M:%S.%f'), timezone_from_name(value['timezone'])
+        )
 
     @classmethod
     def serialize_field(cls, value, field_type):
@@ -535,8 +565,6 @@ class JobInfo(DefaultFieldsAttributeDict):  # pylint: disable=too-many-instance-
 
         :return: A string with serialised representation of the current data.
         """
-        from aiida.common import json
-
         return json.dumps(self.get_dict())
 
     def get_dict(self):
@@ -566,6 +594,4 @@ class JobInfo(DefaultFieldsAttributeDict):  # pylint: disable=too-many-instance-
 
         :param data: The string with the JSON-serialised data to load from
         """
-        from aiida.common import json
-
         return cls.load_from_dict(json.loads(data))
