@@ -10,17 +10,21 @@
 """ provides functionality to create graphs of the AiiDa data providence,
 *via* graphviz.
 """
-
 import os
 from types import MappingProxyType  # pylint: disable=no-name-in-module,useless-suppression
+from typing import TYPE_CHECKING, Optional
 
 from graphviz import Digraph
 
 from aiida import orm
 from aiida.common import LinkType
+from aiida.manage import get_manager
 from aiida.orm.utils.links import LinkPair
 from aiida.tools.graph.graph_traversers import traverse_graph
 
+if TYPE_CHECKING:
+    from aiida.orm.implementation import StorageBackend
+
 __all__ = ('Graph', 'default_link_styles', 'default_node_styles', 'pstate_node_styles', 'default_node_sublabels')
 
 
@@ -95,7 +99,7 @@ def default_node_styles(node):
         }
 
     node_type_map = {
-        'data.code.Code.': {
+        'data.core.code.Code.': {
             'shape': 'ellipse',
             'style': 'filled',
             'fillcolor': '#4ca4b9aa',  # blue
@@ -212,23 +216,23 @@ def default_node_sublabels(node):
     # pylint: disable=too-many-branches
 
     class_node_type = node.class_node_type
-    if class_node_type == 'data.int.Int.':
-        sublabel = f"value: {node.get_attribute('value', '')}"
-    elif class_node_type == 'data.float.Float.':
-        sublabel = f"value: {node.get_attribute('value', '')}"
-    elif class_node_type == 'data.str.Str.':
-        sublabel = f"{node.get_attribute('value', '')}"
-    elif class_node_type == 'data.bool.Bool.':
-        sublabel = f"{node.get_attribute('value', '')}"
-    elif class_node_type == 'data.code.Code.':
+    if class_node_type == 'data.core.int.Int.':
+        sublabel = f"value: {node.base.attributes.get('value', '')}"
+    elif class_node_type == 'data.core.float.Float.':
+        sublabel = f"value: {node.base.attributes.get('value', '')}"
+    elif class_node_type == 'data.core.str.Str.':
+        sublabel = f"{node.base.attributes.get('value', '')}"
+    elif class_node_type == 'data.core.bool.Bool.':
+        sublabel = f"{node.base.attributes.get('value', '')}"
+    elif class_node_type == 'data.core.code.Code.':
         sublabel = f'{os.path.basename(node.get_execname())}@{node.computer.label}'
-    elif class_node_type == 'data.singlefile.SinglefileData.':
+    elif class_node_type == 'data.core.singlefile.SinglefileData.':
         sublabel = node.filename
-    elif class_node_type == 'data.remote.RemoteData.':
+    elif class_node_type == 'data.core.remote.RemoteData.':
         sublabel = f'@{node.computer.label}'
-    elif class_node_type == 'data.structure.StructureData.':
+    elif class_node_type == 'data.core.structure.StructureData.':
         sublabel = node.get_formula()
-    elif class_node_type == 'data.cif.CifData.':
+    elif class_node_type == 'data.core.cif.CifData.':
         formulae = [str(f).replace(' ', '') for f in node.get_formulae() or []]
         sg_numbers = [str(s) for s in node.get_spacegroup_numbers() or []]
         sublabel_lines = []
@@ -237,8 +241,8 @@ def default_node_sublabels(node):
         if sg_numbers:
             sublabel_lines.append(', '.join(sg_numbers))
         sublabel = '; '.join(sublabel_lines)
-    elif class_node_type == 'data.upf.UpfData.':
-        sublabel = f"{node.get_attribute('element', '')}"
+    elif class_node_type == 'data.core.upf.UpfData.':
+        sublabel = f"{node.base.attributes.get('element', '')}"
     elif isinstance(node, orm.ProcessNode):
         sublabel = []
         if node.process_state is not None:
@@ -359,7 +363,8 @@ class Graph:
         link_style_fn=None,
         node_style_fn=None,
         node_sublabel_fn=None,
-        node_id_type='pk'
+        node_id_type='pk',
+        backend: Optional['StorageBackend'] = None
     ):
         """a class to create graphviz graphs of the AiiDA node provenance
 
@@ -398,10 +403,16 @@ class Graph:
         self._node_styles = node_style_fn or default_node_styles
         self._node_sublabels = node_sublabel_fn or default_node_sublabels
         self._node_id_type = node_id_type
+        self._backend = backend or get_manager().get_profile_storage()
 
         self._ignore_node_style = _OVERRIDE_STYLES_DICT['ignore_node']
         self._origin_node_style = _OVERRIDE_STYLES_DICT['origin_node']
 
+    @property
+    def backend(self) -> 'StorageBackend':
+        """The backend used to create the graph"""
+        return self._backend
+
     @property
     def graphviz(self):
         """return a copy of the graphviz.Digraph"""
@@ -417,16 +428,17 @@ class Graph:
         """return a copy of the edges"""
         return self._edges.copy()
 
-    @staticmethod
-    def _load_node(node):
+    def _load_node(self, node):
         """ load a node (if not already loaded)
 
         :param node: node or node pk/uuid
         :type node: int or str or aiida.orm.nodes.node.Node
         :returns: aiida.orm.nodes.node.Node
         """
-        if isinstance(node, (int, str)):
-            return orm.load_node(node)
+        if isinstance(node, int):
+            return orm.Node.collection(self._backend).get(pk=node)
+        if isinstance(node, str):
+            return orm.Node.collection(self._backend).get(uuid=node)
         return node
 
     @staticmethod
@@ -510,7 +522,7 @@ class Graph:
             return None
         if isinstance(link_types, str):
             link_types = [link_types]
-        link_types = tuple([getattr(LinkType, l.upper()) if isinstance(l, str) else l for l in link_types])
+        link_types = tuple(getattr(LinkType, l.upper()) if isinstance(l, str) else l for l in link_types)
         return link_types
 
     def add_incoming(self, node, link_types=(), annotate_links=None, return_pks=True):
@@ -539,10 +551,11 @@ class Graph:
             (node_pk,),
             max_iterations=1,
             get_links=True,
+            backend=self.backend,
             links_backward=valid_link_types,
         )
 
-        traversed_nodes = orm.QueryBuilder().append(
+        traversed_nodes = orm.QueryBuilder(backend=self.backend).append(
             orm.Node,
             filters={'id': {
                 'in': traversed_graph['nodes']
@@ -595,10 +608,11 @@ class Graph:
             (node_pk,),
             max_iterations=1,
             get_links=True,
+            backend=self.backend,
             links_forward=valid_link_types,
         )
 
-        traversed_nodes = orm.QueryBuilder().append(
+        traversed_nodes = orm.QueryBuilder(backend=self.backend).append(
             orm.Node,
             filters={'id': {
                 'in': traversed_graph['nodes']
@@ -634,7 +648,6 @@ class Graph:
         origin_style=MappingProxyType(_OVERRIDE_STYLES_DICT['origin_node']),
         include_process_inputs=False,
         highlight_classes=None,
-        print_func=None
     ):
         """add nodes and edges from an origin recursively,
         following outgoing links
@@ -654,19 +667,8 @@ class Graph:
         :param highlight_classes: target class in exported graph expected to be highlight and
             other nodes are decolorized (Default value = None)
         :typle highlight_classes: tuple of class or class
-        :param print_func:
-            a function to stream information to, i.e. print_func(str)
-            (this feature is deprecated since `v1.1.0` and will be removed in `v2.0.0`)
-
         """
         # pylint: disable=too-many-arguments,too-many-locals
-        import warnings
-        from aiida.common.warnings import AiidaDeprecationWarning
-        if print_func:
-            warnings.warn( # pylint: disable=no-member
-                '`print_func` is deprecated because graph traversal has been refactored', AiidaDeprecationWarning
-            )
-
         # Get graph traversal rules where the given link types and direction are all set to True,
         # and all others are set to False
         origin_pk = self._load_node(origin).pk
@@ -676,6 +678,7 @@ class Graph:
             (origin_pk,),
             max_iterations=depth,
             get_links=True,
+            backend=self.backend,
             links_forward=valid_link_types,
         )
 
@@ -686,13 +689,14 @@ class Graph:
                 traversed_graph['nodes'],
                 max_iterations=1,
                 get_links=True,
+                backend=self.backend,
                 links_backward=[LinkType.INPUT_WORK, LinkType.INPUT_CALC]
             )
             traversed_graph['nodes'] = traversed_graph['nodes'].union(traversed_outputs['nodes'])
             traversed_graph['links'] = traversed_graph['links'].union(traversed_outputs['links'])
 
         # Do one central query for all nodes in the Graph and generate a {id: Node} dictionary
-        traversed_nodes = orm.QueryBuilder().append(
+        traversed_nodes = orm.QueryBuilder(backend=self.backend).append(
             orm.Node,
             filters={'id': {
                 'in': traversed_graph['nodes']
@@ -737,7 +741,6 @@ class Graph:
         origin_style=MappingProxyType(_OVERRIDE_STYLES_DICT['origin_node']),
         include_process_outputs=False,
         highlight_classes=None,
-        print_func=None
     ):
         """add nodes and edges from an origin recursively,
         following incoming links
@@ -757,19 +760,8 @@ class Graph:
         :param highlight_classes:  class label (as displayed in the graph, e.g. 'StructureData', 'FolderData', etc.)
             to be highlight and other nodes are decolorized (Default value = None)
         :typle highlight_classes: list or tuple of str
-        :param print_func: a function to stream information to, i.e. print_func(str)
-
-        .. deprecated:: 1.1.0
-            `print_func` will be removed in `v2.0.0`
         """
         # pylint: disable=too-many-arguments,too-many-locals
-        import warnings
-        from aiida.common.warnings import AiidaDeprecationWarning
-        if print_func:
-            warnings.warn( # pylint: disable=no-member
-                '`print_func` is deprecated because graph traversal has been refactored', AiidaDeprecationWarning
-            )
-
         # Get graph traversal rules where the given link types and direction are all set to True,
         # and all others are set to False
         origin_pk = self._load_node(origin).pk
@@ -779,6 +771,7 @@ class Graph:
             (origin_pk,),
             max_iterations=depth,
             get_links=True,
+            backend=self.backend,
             links_backward=valid_link_types,
         )
 
@@ -789,13 +782,14 @@ class Graph:
                 traversed_graph['nodes'],
                 max_iterations=1,
                 get_links=True,
+                backend=self.backend,
                 links_forward=[LinkType.CREATE, LinkType.RETURN]
             )
             traversed_graph['nodes'] = traversed_graph['nodes'].union(traversed_outputs['nodes'])
             traversed_graph['links'] = traversed_graph['links'].union(traversed_outputs['links'])
 
         # Do one central query for all nodes in the Graph and generate a {id: Node} dictionary
-        traversed_nodes = orm.QueryBuilder().append(
+        traversed_nodes = orm.QueryBuilder(backend=self.backend).append(
             orm.Node,
             filters={'id': {
                 'in': traversed_graph['nodes']
@@ -866,6 +860,7 @@ class Graph:
         self.add_node(origin_node, style_override=dict(origin_style))
 
         query = orm.QueryBuilder(
+            backend=self.backend,
             **{
                 'path': [{
                     'cls': origin_node.__class__,
@@ -926,6 +921,7 @@ class Graph:
             origin_filters = {}
 
         query = orm.QueryBuilder(
+            backend=self.backend,
             **{'path': [{
                 'cls': origin_cls,
                 'filters': origin_filters,
