@@ -8,17 +8,17 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Common cli utilities for transport plugins."""
-import inspect
 from functools import partial
+import inspect
 
 import click
 
-from aiida.cmdline.params import options, arguments
+from aiida.cmdline.params import arguments, options
 from aiida.cmdline.params.options.interactive import InteractiveOption
-from aiida.cmdline.utils.decorators import with_dbenv
 from aiida.cmdline.utils import echo
+from aiida.cmdline.utils.decorators import with_dbenv
 from aiida.common.exceptions import NotExistent
-from aiida.manage.manager import get_manager
+from aiida.manage import get_manager
 
 TRANSPORT_PARAMS = []
 
@@ -38,11 +38,11 @@ def configure_computer_main(computer, user, **kwargs):
     """Configure a computer via the CLI."""
     from aiida import orm
 
-    user = user or orm.User.objects.get_default()
+    user = user or orm.User.collection.get_default()
 
-    echo.echo_info(f'Configuring computer {computer.label} for user {user.email}.')
-    if user.email != get_manager().get_profile().default_user:
-        echo.echo_info('Configuring different user, defaults may not be appropriate.')
+    echo.echo_report(f'Configuring computer {computer.label} for user {user.email}.')
+    if user.email != get_manager().get_profile().default_user_email:
+        echo.echo_report('Configuring different user, defaults may not be appropriate.')
 
     computer.configure(user=user, **kwargs)
     echo.echo_success(f'{computer.label} successfully configured for {user.email}')
@@ -86,11 +86,14 @@ def interactive_default(key, also_non_interactive=False):
         if not also_non_interactive and ctx.params['non_interactive']:
             raise click.MissingParameter()
 
-        user = ctx.params['user'] or orm.User.objects.get_default()
-        computer = ctx.params['computer']
+        user = ctx.params.get('user', None) or orm.User.collection.get_default()
+        computer = ctx.params.get('computer', None)
+
+        if computer is None:
+            return None
 
         try:
-            authinfo = orm.AuthInfo.objects.get(dbcomputer_id=computer.id, aiidauser_id=user.id)
+            authinfo = orm.AuthInfo.collection.get(dbcomputer_id=computer.pk, aiidauser_id=user.pk)
         except NotExistent:
             authinfo = orm.AuthInfo(computer=computer, user=user)
 
