@@ -21,6 +21,7 @@ import paramiko
 from aiida.cmdline.params import options
 from aiida.cmdline.params.types.path import AbsolutePathOrEmptyParamType
 from aiida.common.escaping import escape_for_bash
+
 from ..transport import Transport, TransportInternalError
 
 __all__ = ('parse_sshconfig', 'convert_to_bool', 'SshTransport')
@@ -446,6 +447,7 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
         :raise aiida.common.InvalidOperation: if the channel is already open
         """
         from paramiko.ssh_exception import SSHException
+
         from aiida.common.exceptions import InvalidOperation
         from aiida.transports.util import _DetachedProxyCommand
 
@@ -768,7 +770,7 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
 
         command = f'{rm_exe} {rm_flags} {escape_for_bash(path)}'
 
-        retval, stdout, stderr = self.exec_command_wait(command)
+        retval, stdout, stderr = self.exec_command_wait_bytes(command)
 
         if retval == 0:
             if stderr.strip():
@@ -1224,7 +1226,7 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
         # to simplify writing the above copy function
         command = f'{cp_exe} {cp_flags} {escape_for_bash(src)} {escape_for_bash(dst)}'
 
-        retval, stdout, stderr = self.exec_command_wait(command)
+        retval, stdout, stderr = self.exec_command_wait_bytes(command)
 
         if retval == 0:
             if stderr.strip():
@@ -1272,7 +1274,7 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
         else:
             base_dir = os.path.join(self.getcwd(), path)
 
-        filtered_list = glob.glob(os.path.join(base_dir, pattern))
+        filtered_list = self.glob(os.path.join(base_dir, pattern))
         if not base_dir.endswith('/'):
             base_dir += '/'
         return [re.sub(base_dir, '', i) for i in filtered_list]
@@ -1372,7 +1374,7 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
 
         return stdin, stdout, stderr, channel
 
-    def exec_command_wait(self, command, stdin=None, combine_stderr=False, bufsize=-1):  # pylint: disable=arguments-differ
+    def exec_command_wait_bytes(self, command, stdin=None, combine_stderr=False, bufsize=-1):  # pylint: disable=arguments-differ, too-many-branches
         """
         Executes the specified command and waits for it to finish.
 
@@ -1384,35 +1386,101 @@ class SshTransport(Transport):  # pylint: disable=too-many-public-methods
         :param bufsize: same meaning of paramiko.
 
         :return: a tuple with (return_value, stdout, stderr) where stdout and stderr
-            are strings.
+            are both bytes and the return_value is an int.
         """
+        import socket
+        import time
+
         ssh_stdin, stdout, stderr, channel = self._exec_command_internal(command, combine_stderr, bufsize=bufsize)
 
         if stdin is not None:
             if isinstance(stdin, str):
                 filelike_stdin = io.StringIO(stdin)
-            else:
+            elif isinstance(stdin, bytes):
+                filelike_stdin = io.BytesIO(stdin)
+            elif isinstance(stdin, (io.BufferedIOBase, io.TextIOBase)):
+                # It seems both StringIO and BytesIO work correctly when doing ssh_stdin.write(line)?
+                # (The ChannelFile is opened with mode 'b', but until now it always has been a StringIO)
                 filelike_stdin = stdin
+            else:
+                raise ValueError('You can only pass strings, bytes, BytesIO or StringIO objects')
 
-            try:
-                for line in filelike_stdin.readlines():
-                    ssh_stdin.write(line)
-            except AttributeError:
-                raise ValueError('stdin can only be either a string of a file-like object!')
+            for line in filelike_stdin:
+                ssh_stdin.write(line)
 
         # I flush and close them anyway; important to call shutdown_write
         # to avoid hangouts
         ssh_stdin.flush()
         ssh_stdin.channel.shutdown_write()
 
+        # Now I get the output
+        stdout_bytes = []
+        stderr_bytes = []
+        # 100kB buffer (note that this should be smaller than the window size of paramiko)
+        # Also, apparently if the data is coming slowly, the read() command will not unlock even for
+        # times much larger than the timeout. Therefore we don't want to have very large buffers otherwise
+        # you risk that a lot of output is sent to both stdout and stderr, and stderr goes beyond the
+        # window size and blocks.
+        # Note that this is different than the bufsize of paramiko.
+        internal_bufsize = 100 * 1024
+
+        # Set a small timeout on the channels, so that if we get data from both
+        # stderr and stdout, and the connection is slow, we interleave the receive and don't hang
+        # NOTE: Timeouts and sleep time below, as well as the internal_bufsize above, have been benchmarked
+        # to try to optimize the overall throughput. I could get ~100MB/s on a localhost via ssh (and 3x slower
+        # if compression is enabled).
+        # It's important to mention that, for speed benchmarks, it's important to disable compression
+        # in the SSH transport settings, as it will cap the max speed.
+        stdout.channel.settimeout(0.01)
+        stderr.channel.settimeout(0.01)  # Maybe redundant, as this could be the same channel.
+
+        while True:
+            chunk_exists = False
+
+            if stdout.channel.recv_ready():  # True means that the next .read call will at least receive 1 byte
+                chunk_exists = True
+                try:
+                    piece = stdout.read(internal_bufsize)
+                    stdout_bytes.append(piece)
+                except socket.timeout:
+                    # There was a timeout: I continue as there should still be data
+                    pass
+
+            if stderr.channel.recv_stderr_ready():  # True means that the next .read call will at least receive 1 byte
+                chunk_exists = True
+                try:
+                    piece = stderr.read(internal_bufsize)
+                    stderr_bytes.append(piece)
+                except socket.timeout:
+                    # There was a timeout: I continue as there should still be data
+                    pass
+
+            # If chunk_exists, there is data (either already read and put in the std*_bytes lists, or
+            # still in the buffer because of a timeout). I need to loop.
+            # Otherwise, there is no data in the buffers, and I enter this block.
+            if not chunk_exists:
+                # Both channels have no data in the buffer
+                if channel.exit_status_ready():
+                    # The remote execution is over
+
+                    # I think that in some corner cases there might still be some data,
+                    # in case the data arrived between the previous calls and this check.
+                    # So we do a final read. Since the execution is over, I think all data is in the buffers,
+                    # so we can just read the whole buffer without loops
+                    stdout_bytes.append(stdout.read())
+                    stderr_bytes.append(stderr.read())
+                    # And we go out of the `while True` loop
+                    break
+                # The exit status is not ready:
+                # I just put a small sleep to avoid infinite fast loops when data
+                # is not available on a slow connection, and loop
+                time.sleep(0.01)
+
         # I get the return code (blocking)
+        # However, if I am here, the exit status is ready so this should be returning very quickly
         retval = channel.recv_exit_status()
 
-        # needs to be after 'recv_exit_status', otherwise it might hang
-        output_text = stdout.read().decode('utf-8')
-        stderr_text = stderr.read().decode('utf-8')
-
-        return retval, output_text, stderr_text
+        return (retval, b''.join(stdout_bytes), b''.join(stderr_bytes))
 
     def gotocomputer_command(self, remotedir):
         """
